@@ -2,9 +2,9 @@
 
 __author__ = "Rob Knight, Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME Project"
-__credits__ = ["Rob Knight", "Greg Caporaso", "Kyle Bittinger", "Antonio Gonzalez Pena"]
+__credits__ = ["Rob Knight", "Greg Caporaso", "Kyle Bittinger", "Antonio Gonzalez Pena", "David Soergel"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -25,7 +25,7 @@ from qiime.util import get_tmp_filename
 from cogent.app.formatdb import build_blast_db_from_fasta_path
 from cogent.app.blast import blast_seqs, Blastall, BlastResult
 from qiime.pycogent_backports import rdp_classifier
-from qiime.pycogent_backports import rdp_classifier20
+from qiime.pycogent_backports import rtax
 from cogent.parse.fasta import MinimalFastaParser
 from qiime.util import FunctionWithParams, get_rdp_jarpath
 
@@ -35,47 +35,22 @@ from qiime.util import FunctionWithParams, get_rdp_jarpath
 This module has the responsibility for taking a set of sequences and
 providing a taxon assignment for each sequence."""
 
-def check_rdp_version(rdp_jar_path,requested_version):
-    return requested_version in rdp_jar_path
-
-def error_on_bad_rdp_version(option_parser,assignment_method):
+def guess_rdp_version(rdp_jarpath=None):
+    if rdp_jarpath is None:
         rdp_jarpath = get_rdp_jarpath()
-        if rdp_jarpath == None:
-            option_parser.error("RDP classifier is not installed or "
-             "not accessible to QIIME. See install instructions here: "
-             "http://qiime.org/install/install.html#rdp-install")
-        elif assignment_method == 'rdp':
-            if not check_rdp_version(rdp_jarpath,"2.2"):
-                option_parser.error("Specified RDP version 2.2 (default), "
-                "but that version is not installed. Pass -m rdp20 for RDP "
-                "classifier versions 2.0 and 2.0.1.")
-        elif assignment_method == 'rdp20':
-            if not check_rdp_version(rdp_jarpath,"2.0"):
-                option_parser.error("Specified RDP version 2.0 (default), "
-                "but that version is not installed. Pass -m rdp for RDP "
-                "classifier versions 2.2.")
-        else:
-            # not possible to get here, but don't like if/elif without else
-            raise ValueError,\
-             "Unknown assignment method: %s" % assignment_method
-
-def guess_rdp_version():
-    rdp_jarpath = get_rdp_jarpath()
-    if rdp_jarpath == None:
-        raise ValueError, \
-         ("RDP classifier is not installed or "
-          "not accessible to QIIME. See install instructions here: "
-          "http://qiime.org/install/install.html#rdp-install")
+    if rdp_jarpath is None:
+        raise ValueError(
+            "RDP classifier is not installed or "
+            "not accessible to QIIME. See install instructions here: "
+            "http://qiime.org/install/install.html#rdp-install")
     elif "2.2" in rdp_jarpath:
         return "rdp22"
-    elif "2.0" in rdp_jarpath:
-        return "rdp20"
     else:
-        raise ValueError,\
-         ("Can't determine RDP version number. Only versions 2.0, 2.0.1,"
-          " and 2.2 are supported by QIIME. RDP jar path is:"
-          " %s" % rdp_jarpath)
-    
+        raise ValueError(
+            "RDP classifier filename does not look like version 2.2.  Only "
+            "version 2.2 is supported by QIIME. RDP jar path is:"
+            " %s" % rdp_jarpath)
+
 
 class TaxonAssigner(FunctionWithParams):
     """A TaxonAssigner assigns a taxon to each of a set of sequences.
@@ -87,9 +62,9 @@ class TaxonAssigner(FunctionWithParams):
 
     def __init__(self, params):
         """Return new TaxonAssigner object with specified params.
-        
+
         Note: expect params to contain both generic and per-method (e.g. for
-        RDP classifier w/ Hugenholtz taxonomy) params, so leaving it as a dict 
+        RDP classifier w/ Hugenholtz taxonomy) params, so leaving it as a dict
         rather than setting attributes. Some standard entries in params are:
 
         Taxonomy: taxonomy used (e.g. RDP, Hugenholtz)
@@ -101,7 +76,7 @@ class TaxonAssigner(FunctionWithParams):
 
     def __call__ (self, seq_path, result_path=None, log_path=None):
         """Returns dict mapping {seq_id:(taxonomy, confidence)} for each seq.
-        
+
         Parameters:
         seq_path: path to file of sequences
         result_path: path to file of results. If specified, should
@@ -120,7 +95,7 @@ class TaxonAssigner(FunctionWithParams):
             if line:
                 identifier, taxonomy = map(strip, line.split('\t'))
                 result[identifier] = taxonomy
-        return result 
+        return result
 
 
 class BlastTaxonAssigner(TaxonAssigner):
@@ -138,19 +113,19 @@ class BlastTaxonAssigner(TaxonAssigner):
             }
         _params.update(params)
         TaxonAssigner.__init__(self, _params)
-    
+
     def __call__(self, seq_path=None, seqs=None, result_path=None, log_path=None):
         """Returns dict mapping {seq_id:(taxonomy, confidence)} for each seq.
         """
         assert seq_path or seqs, \
          "Must provide either seqs or seq_path when calling a BlastTaxonAssigner."
-         
+
         # initialize the logger
         logger = self._get_logger(log_path)
         logger.info(str(self))
-        
+
         # assign the blast database, either as a pre-exisiting database
-        # specified as self.Params['blast_db'] or by creating a 
+        # specified as self.Params['blast_db'] or by creating a
         # temporary database from the sequence file specified
         # as self.Params['reference_seqs_filepath']
         try:
@@ -161,38 +136,38 @@ class BlastTaxonAssigner(TaxonAssigner):
             refseqs_dir, refseqs_name = split(reference_seqs_path)
             blast_db, db_files_to_remove = \
              build_blast_db_from_fasta_path(reference_seqs_path)
-        
-        # build the mapping of sequence identifier 
+
+        # build the mapping of sequence identifier
         # (wrt to the blast db seqs) to taxonomy
         id_to_taxonomy_map = self._parse_id_to_taxonomy_file(\
-         open(self.Params['id_to_taxonomy_filepath'],'U')) 
-        
-        ## Iterate over the input self.SeqsPerBlastRun seqs at a time. 
+         open(self.Params['id_to_taxonomy_filepath'],'U'))
+
+        ## Iterate over the input self.SeqsPerBlastRun seqs at a time.
         # There are two competing issues here when dealing with very large
         # inputs. If all sequences are read in at once, the containing object
         # can be very large, causing the system to page. On the other hand,
-        # in such cases it would be very slow to treat each sequence 
+        # in such cases it would be very slow to treat each sequence
         # individually, since blast requires a filepath. Each call would
         # therefore involve writing a single sequence to file, opening/closing
         # and removing the file. To balance this, sequences are read in and
         # blasted in chunks of self.SeqsPerBlastRun (defualt: 1000) at a time.
         # This appears to solve the problem with the largest sets I've worked
-        # with so far. 
-        
+        # with so far.
+
         if seq_path:
             # Get a seq iterator
             seqs = MinimalFastaParser(open(seq_path))
         # Build object to keep track of the current set of sequence to be
-        # blasted, and the results (i.e., seq_id -> (taxonomy,quaility score) 
+        # blasted, and the results (i.e., seq_id -> (taxonomy,quaility score)
         # mapping)
         current_seqs = []
         result = {}
-        
+
         # Iterate over the (seq_id, seq) pairs
         for seq_id, seq in seqs:
             # append the current seq_id,seq to list of seqs to be blasted
             current_seqs.append((seq_id,seq))
-            
+
             # When there are 1000 in the list, blast them
             if len(current_seqs) == self.SeqsPerBlastRun:
                 # update the result object
@@ -203,8 +178,8 @@ class BlastTaxonAssigner(TaxonAssigner):
         # Assign taxonomy to the remaining sequences
         result.update(self._seqs_to_taxonomy(\
          current_seqs,blast_db,id_to_taxonomy_map))
-        ## End iteration over the input self.SeqsPerBlastRun seqs at a time. 
-        
+        ## End iteration over the input self.SeqsPerBlastRun seqs at a time.
+
         # Write log data if we have a path (while the logger can handle
         # being called if we are not logging, some of these steps are slow).
         if log_path is not None:
@@ -214,11 +189,11 @@ class BlastTaxonAssigner(TaxonAssigner):
             logger.info('Number with no blast hits: %s' % num_null_hits)
 
         if result_path:
-            # if the user provided a result_path, write the 
+            # if the user provided a result_path, write the
             # results to file
             of = open(result_path,'w')
             for seq_id, (lineage, confidence, blast_hit_id) in result.items():
-                of.write('%s\t%s\t%s\t%s\n' % 
+                of.write('%s\t%s\t%s\t%s\n' %
                  (seq_id, lineage, confidence, blast_hit_id))
             of.close()
             result = None
@@ -227,7 +202,7 @@ class BlastTaxonAssigner(TaxonAssigner):
             # Returning the data as a dict, so no modification to result
             # is necessary.
             pass
-                 
+
             # if no result_path was provided, return the data as a dict
             logger.info('Result path: None, returned as dict.')
 
@@ -237,19 +212,19 @@ class BlastTaxonAssigner(TaxonAssigner):
 
         # return the result
         return result
-        
+
     def _seqs_to_taxonomy(self,seqs,blast_db,id_to_taxonomy_map):
         """ Assign taxonomy to (seq_id,seq) pairs
         """
         # Handle the case of no seqs passed in
-        if not seqs: 
+        if not seqs:
             return {}
         # blast the seqs
         blast_hits = self._get_blast_hits(blast_db,seqs)
 
         # select the best blast hit for each query sequence
         best_blast_hit_ids = self._get_first_blast_hit_per_seq(blast_hits)
- 
+
         # map the identifier of the best blast hit to (taxonomy, e-value)
         return self._map_ids_to_taxonomy(\
              best_blast_hit_ids,id_to_taxonomy_map)
@@ -272,14 +247,14 @@ class BlastTaxonAssigner(TaxonAssigner):
         for query_id, hit in hits.items():
             query_id=query_id.split()[0]
             try:
-                hit_id, e_value = hit 
+                hit_id, e_value = hit
                 hits[query_id] = \
                   (id_to_taxonomy_map.get(hit_id, None),e_value,hit_id)
             except TypeError:
                 hits[query_id] = ('No blast hit', None, None)
 
         return hits
-        
+
     def _get_blast_hits(self,blast_db,seqs):
         """ blast each seq in seqs against blast_db and retain good hits
         """
@@ -287,18 +262,18 @@ class BlastTaxonAssigner(TaxonAssigner):
         min_percent_identity = self.Params['Min percent identity']
         seq_ids = [s[0] for s in seqs]
         result = {}
-        
+
         blast_result = blast_seqs(\
          seqs,Blastall,blast_db=blast_db,\
          params={'-p':'blastn','-n':'T'},\
          add_seq_names=False)
-         
+
         if blast_result['StdOut']:
             lines = [x for x in blast_result['StdOut']]
             blast_result = BlastResult(lines)
         else:
             return {}.fromkeys(seq_ids,[])
-            
+
         for seq_id in seq_ids:
             blast_result_id = seq_id.split()[0]
             try:
@@ -310,7 +285,7 @@ class BlastTaxonAssigner(TaxonAssigner):
                 result[seq_id] = []
 
         return result
-        
+
     def _get_first_blast_hit_per_seq(self,blast_hits):
         """ discard all blast hits except the best for each query sequence
         """
@@ -320,10 +295,10 @@ class BlastTaxonAssigner(TaxonAssigner):
             try:
                 result[k] = v[0]
             except IndexError:
-                # If there is no good blast hit, do we want to 
+                # If there is no good blast hit, do we want to
                 # leave the key out, or have it point to None?
                 result[k] = None
-        
+
         return result
 
 
@@ -335,10 +310,10 @@ class RdpTaxonAssigner(TaxonAssigner):
     Citation = "Wang, Q, G. M. Garrity, J. M. Tiedje, and J. R. Cole. 2007. Naive Bayesian Classifier for Rapid Assignment of rRNA Sequences into the New Bacterial Taxonomy. Appl Environ Microbiol. 73(16):5261-7."
     Taxonomy = "RDP"
     _tracked_properties = ['Application','Citation','Taxonomy']
-    
+
     def __init__(self, params):
         """Return new RdpTaxonAssigner object with specified params.
-        
+
         Standard entries in params are:
 
         Taxonomy: taxonomy used (e.g. RDP, Hugenholtz)
@@ -364,26 +339,26 @@ class RdpTaxonAssigner(TaxonAssigner):
     def __call__(self, seq_path, result_path=None, log_path=None):
         """Returns dict mapping {seq_id:(taxonomy, confidence)} for
         each seq.
-        
+
         Parameters:
-        seq_path: path to file of sequences 
-        result_path: path to file of results. If specified, dumps the 
+        seq_path: path to file of sequences
+        result_path: path to file of results. If specified, dumps the
             result to the desired path instead of returning it.
         log_path: path to log, which should include dump of params.
         """
-        
+
         min_conf = self.Params['Confidence']
         training_data_properties_fp = self.Params['training_data_properties_fp']
         reference_sequences_fp = self.Params['reference_sequences_fp']
         id_to_taxonomy_fp = self.Params['id_to_taxonomy_fp']
         max_memory = self.Params['max_memory']
-        
+
         seq_file = open(seq_path, 'r')
         if reference_sequences_fp and id_to_taxonomy_fp:
             # Train and assign taxonomy
             taxonomy_file, training_seqs_file = self._generate_training_files()
             results = self._train_fcn(
-                training_seqs_file, taxonomy_file, seq_file, 
+                training_seqs_file, taxonomy_file, seq_file,
                 min_confidence=min_conf,
                 classification_output_fp=result_path,
                 max_memory=max_memory)
@@ -470,7 +445,7 @@ class RdpTrainingSet(object):
     def get_training_seqs(self):
         """Returns an iterator of valid training sequences in
         RDP-compatible format
-        
+
         Each training sequence is represented by a tuple (rdp_id,
         seq).  The rdp_id consists of two items: the original sequence
         ID with whitespace replaced by underscores, and the lineage
@@ -532,7 +507,7 @@ class RdpTree(object):
 
     def insert_lineage(self, lineage):
         """Inserts an assignment into the taxonomic tree.
-        
+
         Lineage must support the iterator interface, or provide an
         __iter__() method that returns an iterator.
         """
@@ -542,7 +517,7 @@ class RdpTree(object):
             if taxon not in self.children:
                 self.children[taxon] = self.__class__(
                     name=taxon, parent=self, counter=self.counter)
-            retval = self.children[taxon].insert_lineage(lineage)            
+            retval = self.children[taxon].insert_lineage(lineage)
         except StopIteration:
             retval = self
         return retval
@@ -596,17 +571,79 @@ class RdpTree(object):
         return taxonomy_str
 
 
-class Rdp20TaxonAssigner(RdpTaxonAssigner):
-    Name = "Rdp20TaxonAssigner"
-    Application = "RDP classfier, version 2.0"
-    
-    @property
-    def _assign_fcn(self):
-        return rdp_classifier20.assign_taxonomy
-
-    @property
-    def _train_fcn(self):
-        return rdp_classifier20.train_rdp_classifier_and_assign_taxonomy
-
-
 _QIIME_RDP_TAXON_TAG = "_qiime_unique_taxon_tag_"
+
+
+class RtaxTaxonAssigner(TaxonAssigner):
+    """Assign taxon using RTAX
+    """
+    Name = "RtaxTaxonAssigner"
+    Application = "RTAX classifier" # ", version 0.98"  # don't hardcode the version number, as it may change, and then the log output test would fail
+    Citation = "Soergel D.A.W., Dey N., Knight R., and Brenner S.E.  2012.  Selection of primers for optimal taxonomic classification of environmental 16S rRNA gene sequences.  ISME J."
+    _tracked_properties = ['Application','Citation']
+
+    def __init__(self, params):
+        """Return new RtaxTaxonAssigner object with specified params.
+        """
+        _params = {
+            'id_to_taxonomy_fp': None,
+            'reference_sequences_fp': None,
+            # 'delimiter': ","
+            'header_id_regex' : "\\S+\\s+(\\S+?)\/",  # use the amplicon ID, not including /1 or /3, as the primary key for the query sequences
+            'read_id_regex' : "\\S+\\s+(\\S+)",  # OTU clustering produces ">clusterID read_1_id"
+            'amplicon_id_regex' : "(\\S+)\\s+(\\S+?)\/",  # split_libraries produces >read_1_id ampliconID/1 .   This makes a map between read_1_id and ampliconID.
+            'read_1_seqs_fp' : None,
+            'read_2_seqs_fp' : None,
+            'single_ok' : False
+            }
+        _params.update(params)
+        TaxonAssigner.__init__(self, _params)
+
+    def __call__(self, seq_path, result_path=None, log_path=None):
+        """Returns dict mapping {seq_id:(taxonomy, confidence)} for
+        each seq.
+
+        Parameters:
+        seq_path: path to file of sequences
+        result_path: path to file of results. If specified, dumps the
+            result to the desired path instead of returning it.
+        log_path: path to log, which should include dump of params.
+        """
+
+        if log_path:
+            self.writeLog(log_path)
+
+        reference_sequences_fp = self.Params['reference_sequences_fp']
+        assert reference_sequences_fp, \
+            "Must provide reference_sequences_fp when calling an RtaxTaxonAssigner."
+
+        id_to_taxonomy_fp = self.Params['id_to_taxonomy_fp']
+        assert id_to_taxonomy_fp, \
+            "Must provide id_to_taxonomy_fp when calling an RtaxTaxonAssigner."
+
+        # delimiter = self.Params['delimiter']
+        read_1_seqs_fp=self.Params['read_1_seqs_fp']
+        assert read_1_seqs_fp, \
+            "Must provide read_1_seqs_fp when calling an RtaxTaxonAssigner."
+
+        # following params may all be null
+
+        read_2_seqs_fp=self.Params['read_2_seqs_fp']
+        single_ok=self.Params['single_ok']
+        header_id_regex=self.Params['header_id_regex']
+        assert header_id_regex, \
+            "Must not provide empty header_id_regex when calling an RtaxTaxonAssigner; leave unset"\
+            "to use default if in doubt."
+
+        read_id_regex=self.Params['read_id_regex']
+        amplicon_id_regex=self.Params['amplicon_id_regex']
+
+        # seq_file = open(seq_path, 'r')
+        results = rtax.assign_taxonomy(seq_path, reference_sequences_fp, id_to_taxonomy_fp,
+                                       read_1_seqs_fp, read_2_seqs_fp, single_ok=single_ok,
+                                       header_id_regex=header_id_regex, read_id_regex=read_id_regex,
+                                       amplicon_id_regex=amplicon_id_regex, output_fp=result_path,
+                                       log_path=log_path)
+
+
+        return results
