@@ -5,7 +5,7 @@ __author__ = "Jesse Stombaugh"
 __copyright__ = "Copyright 2011, The QIIME Project" #consider project name
 __credits__ = ["Rob Knight","Jesse Stombaugh"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Jesse Stombaugh"
 __email__ = "jesse.stombaugh@colorado.edu"
 __status__ = "Release"
@@ -13,8 +13,7 @@ __status__ = "Release"
 """Code for coloring series based on prefs file.
 """
 from colorsys import rgb_to_hsv, hsv_to_rgb
-from parse import parse_mapping_file, group_by_field, parse_otu_table,\
-                  parse_taxa_summary_table
+from parse import parse_mapping_file, group_by_field, parse_taxa_summary_table
 from numpy import array
 import os
 import re
