@@ -6,7 +6,7 @@ __author__ = "Jens Reeder"
 __copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Jens Reeder", "Rob Knight", "Nigel Cook"]#remember to add yourself if you make changes
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Jens Reeder"
 __email__ = "jens.reeder@gmail.com"
 __status__ = "Release"
@@ -19,7 +19,7 @@ from random import sample
 
 from asynchat import async_chat
 from socket import socket, AF_INET, SOCK_STREAM, gethostname, error
-
+from cogent.util.misc import app_path
 from cogent.app.util import ApplicationNotFoundError
 from qiime.util import load_qiime_config, get_qiime_scripts_dir
 
@@ -30,18 +30,21 @@ def submit_jobs(commands, prefix):
 
     prefix: A uniq prefix used to name submit script
 """
-
     qiime_config = load_qiime_config()
     CLUSTER_JOBS_SCRIPT = qiime_config['cluster_jobs_fp']
 
-    if not CLUSTER_JOBS_SCRIPT or not exists(CLUSTER_JOBS_SCRIPT):
+    if not CLUSTER_JOBS_SCRIPT:
         raise ApplicationNotFoundError,"cluster_jobs_fp not set in config file!"
+    if not (exists(CLUSTER_JOBS_SCRIPT) or app_path(CLUSTER_JOBS_SCRIPT)):
+        raise ApplicationNotFoundError,"cluster_jobs_fp not in $PATH or provided as full path!"
+        
 
     outfilename = prefix+"_commands.txt"
     fh = open(outfilename, "w") 
     fh.write("\n".join(commands))
     fh.close()
-    system('%s -ms %s %s'%(CLUSTER_JOBS_SCRIPT, outfilename, prefix))
+    cmd = '%s -ms %s %s'%(CLUSTER_JOBS_SCRIPT, outfilename, prefix)
+    system(cmd)
     remove(outfilename)
 
 def setup_workers(num_cpus, outdir, server_socket, verbose=True,
@@ -82,6 +85,7 @@ def setup_workers(num_cpus, outdir, server_socket, verbose=True,
         else :
             cmd  = "%s %s -f %s -s %s -p %s" % (qiime_config['python_exe_fp'],
                                             DENOISE_WORKER, name, host, port)
+
         if verbose:
             cmd += " -v"
         if error_profile:
