@@ -6,7 +6,7 @@ __credits__ = ["Rob Knight", "Justin Kuczynski","Jeremy Widmann", \
         "Antonio Gonzalez Pena", "Daniel McDonald"] 
 #remember to add yourself if you make changes
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -16,6 +16,8 @@ from numpy import isnan, log10, median
 from StringIO import StringIO
 from cogent import Sequence
 from re import compile, sub
+from biom.table import DenseOTUTable
+from qiime.util import get_qiime_library_version
 
 """Contains formatters for the files we expect to encounter in 454 workflow.
 
@@ -58,7 +60,7 @@ def format_p_value_for_num_iters(p,num_iters):
         # this can be the last step of a long process, so we don't 
         # want to fail
         return "Too few iters to compute p-value (num_iters=%d)" % num_iters
-    decimal_places = int(log10(num_iters))
+    decimal_places = int(log10(num_iters+1))
     result = ('%1.'+'%df' % decimal_places) % p
     return result
 
@@ -191,7 +193,7 @@ def format_matrix(data, row_names, col_names):
             raise ValueError, "Unsupported data type for format_matrix"
 
     lines = []
-    row_names = map(str, row_names)   
+    row_names = map(str, row_names) 
     col_names = map(str, col_names)   
     #just in case they weren't strings initially
     lines.append('\t'.join([''] + col_names))
@@ -201,58 +203,18 @@ def format_matrix(data, row_names, col_names):
 
 def format_otu_table(sample_names, otu_names, data, taxonomy=None,
     comment=None, skip_empty=False,legacy=True):
-    """Writes OTU table as tab-delimited text.
-    
-    inputs: sample_names, otu_names are lists of strings
-    data is numpy 2d array, num_otus x num_samples
-    taxonomy is list of length = num_otus
-    
-    legacy: write 'legacy' format otu table -- these are
-     the pre-Qiime 1.2.0-dev OTU tables. This is True by
-     default, until after the 1.2.0 release.
-    
+    """Returns string representing OTU table as biom file
     """
-    lines = []
-    if comment == None:
-        comment_line = " QIIME v%s OTU table" % __version__
-    else:
-        comment_line = str(comment)
-        
-    if legacy:
-        otu_id_s = "#OTU ID"
-    else:
-        otu_id_s = "OTU ID"
-    lines.append('#'+comment_line)
-            
-    if data.shape != (len(otu_names), len(sample_names)):
-        raise ValueError, "Data shape of %s doesn't match %s OTUs, %s samples" \
-            % (data.shape, len(otu_names), len(sample_names))
-    #data = numpy.array(data, dtype='str') ##BAD! truncates some ints!
-    
-    sample_names = map(str, sample_names)
-    otu_names = map(str, otu_names)
-    
-    if taxonomy:
-        lines.append('\t'.join([otu_id_s] + sample_names + 
-            ['Consensus Lineage']))
-        for otu_name, vals, taxon in zip(otu_names, data, taxonomy):
-            if (skip_empty and filter(lambda a: a!=0, vals)==[]):
-                #skip otu with zero counts
-                continue
-            if isinstance(taxon, str):
-                pass # taxon string will be added to row
-            elif hasattr(taxon, '__iter__'):
-                taxon = ';'.join(taxon) # taxon is now a string
-            else:
-                raise TypeError, "unrecognized taxonomy format" +\
-                    ", try a list of strings"
-            lines.append('\t'.join([otu_name] + map(str, vals.tolist()) + [taxon]))
-    else:
-        lines.append('\t'.join([otu_id_s] + sample_names))
-        for otu_name, vals in zip(otu_names, data):
-            lines.append('\t'.join([otu_name] + map(str,vals.tolist())))
-    
-    return '\n'.join(lines)
+    print "Deprecation Warning: you should not be using format_otu_table. Instead use qiime.pycogent_backports.rich_otu_table.table_factory().getBiomFormatJsonString()."
+    if taxonomy != None:
+        def strip_f(s):
+            return s.strip()
+        taxonomy = [{'taxonomy':map(strip_f,t.split(';'))} for t in taxonomy]
+    otu_table = DenseOTUTable(Data=data,
+                              SampleIds=sample_names, 
+                              ObservationIds=otu_names,
+                              ObservationMetadata=taxonomy)
+    return format_biom_table(otu_table)
 
 def format_coords(coord_header, coords, eigvals, pct_var, headers = True):
     """formats coords given specified coords matrix etc."""
@@ -458,12 +420,23 @@ def format_map_file(headers, id_map, desc_key, sample_id_key, \
         result.append('\t'.join(map(str, curr_line)))
     return '\n'.join(result)
     
-def format_histograms(pre_hist, post_hist, bin_edges):
+'''def format_histograms(pre_hist, post_hist, bin_edges):
     """Returns text-formatted histogram."""
     lines = []
     lines.append('Length\tBefore\tAfter')
     for edge, pre, post in zip(bin_edges, pre_hist, post_hist):
         lines.append('\t'.join(map(str, [edge, pre, post])))
+    return '\n'.join(lines)'''
+    
+def format_histograms(raw_hist, pre_hist, post_hist, bin_edges):
+    """Returns text-formatted histogram.  Needs to take 3 bins of data"""
+    lines = []
+    lines.append('# bins raw sequence lengths, length of sequences that ' 
+     'pass quality filters before processing, and lengths of sequences that '
+     'pass quality filters post processing.')
+    lines.append('Length\tRaw\tBefore\tAfter')
+    for edge, raw, pre, post in zip(bin_edges, raw_hist, pre_hist, post_hist):
+        lines.append('\t'.join(map(str, [edge, raw, pre, post])))
     return '\n'.join(lines)
     
 def format_histogram_one_count(counts, bin_edges):
@@ -500,6 +473,12 @@ def format_split_libraries_fastq_log(count_barcode_not_in_map,
     counts.reverse()
     for sequence_count, sample_id in counts:
         log_out.append('%s\t%d' % (sample_id,sequence_count))
+        
+    total_seqs_written = 0
+    for curr_count in counts:
+        total_seqs_written += curr_count[0]
+        
+    log_out.append('\nTotal number seqs written\t%d' % total_seqs_written)
     return '\n'.join(log_out)
 
 def format_unifrac_sample_mapping(sample_ids, otu_ids, otu_table_array):
@@ -571,4 +550,170 @@ def illumina_data_to_fastq(record_data,number_of_bases=None):
     
     return '@%s\n%s\n+\n%s' % (header,seq,qual), pass_filter
 
+def format_biom_table(biom_table):
+    """ Given a biom-format Table object, returns that Table as a BIOM string"""
+    generated_by_str = "QIIME " + get_qiime_library_version()
+    return biom_table.getBiomFormatJsonString(generated_by_str)
+    
+def format_mapping_html_data(header,
+                             mapping_data,
+                             errors,
+                             warnings):
+    """ Generates list of lines of html for displaying problems with metadata
+    
+    This function assumes that errors and warnings each are tab separated with
+    an index in the form of row,column where 0,0 is the SampleID position.  If
+    the position -1,-1 is given, this indicates that the error is something
+    missing entirely from the mapping file (eg., an added demultiplex field 
+    that does not exist).
+    
+    header:  list of header strings
+    mapping_data:  list of lists of raw metadata mapping file data
+    errors:  list of errors, should contain indices 
+    warnings:  list of warnings, should contain indices
+    """
+    
+    html_lines = HTML_LINES_INIT
+    
+    if not errors and not warnings:
+        html_lines += "<h1>No errors or warnings detected.<br></h1>"
+    # Find errors/warnings that are not in particular cells
+    general_errors = ""
+    for curr_err in errors:
+        loc = curr_err.split('\t')[1].strip()
+        if loc == "-1,-1":
+            general_errors += '<td bgcolor="red"><font color="white">' +\
+             curr_err.split('\t')[0] + '<font color="black"></td>'
+    general_warnings = ""
+    for curr_warning in warnings:
+        loc = curr_warning.split('\t')[1].strip()
+        if loc == "-1,-1":
+            general_warnings += '<td bgcolor="yellow">' +\
+             curr_err.split('\t')[0] + "</td>"
+             
+
+    html_lines += HTML_LINES_MSG % ("+-%./ :,;_",
+     general_errors, general_warnings)
+    
+    # Check header fields, color and add popup messages if errors/warnings 
+    # are present
+    formatted_header = ""
+    for curr_field in range(len(header)):
+        all_errs_warnings = ""
+        curr_pos = "%s,%s" % (0, curr_field)
+        for curr_warning in warnings:
+            loc = curr_warning.split('\t')[1].strip()
+            if loc == curr_pos:
+                bg_color = "yellow"
+                font_color = "black"
+                all_errs_warnings += curr_warning.split('\t')[0] + "<br>"
+        
+        for curr_err in errors:
+            loc = curr_err.split('\t')[1].strip()
+            if loc == curr_pos:
+                bg_color = "red"
+                font_color = "white"
+                all_errs_warnings += curr_err.split('\t')[0] + "<br>"
+        if not all_errs_warnings:
+            formatted_header += "<th>%s</th>" % header[curr_field]
+        elif not header[curr_field]:
+            formatted_header += """<th bgcolor=%s><a href="javascript:void(0);" onmouseover="return overlib('%s');" onmouseout="return nd();"><font color=%s>%s</a></th>""" % (bg_color, all_errs_warnings.replace('"',''), font_color, "missing data")
+
+        else:
+            formatted_header += """<th bgcolor=%s><a href="javascript:void(0);" onmouseover="return overlib('%s');" onmouseout="return nd();"><font color=%s>%s</a></th>""" % (bg_color, all_errs_warnings.replace('"', ''), font_color, header[curr_field])
+
+    html_lines += HTML_LINES_HEADER % formatted_header
+    
+    formatted_data = ""
+    correction_ix = 1
+    
+    for curr_row in range(len(mapping_data)):
+        formatted_data += "<tr>"
+        for curr_cell in range(len(mapping_data[curr_row])):
+            all_errs_warnings = ""
+            append_location = False
+            curr_pos = "%s,%s" % (curr_row + correction_ix, curr_cell)
+            for curr_warning in warnings:
+                loc = curr_warning.split('\t')[1].strip()
+                if loc == curr_pos:
+                    append_location = True
+                    bg_color = "yellow"
+                    font_color = "black"
+                    all_errs_warnings += curr_warning.split('\t')[0] + "<br>"
+                    
+            for curr_err in errors:
+                loc = curr_err.split('\t')[1].strip()
+                if loc == curr_pos:
+                    append_location = True
+                    bg_color = "red"
+                    font_color = "white"
+                    all_errs_warnings += curr_err.split('\t')[0] + "<br>"
+            if append_location:
+                if len(mapping_data[curr_row][0]) == 0:
+                    sample_id_name = "missing sample id"
+                else:
+                    sample_id_name = mapping_data[curr_row][0]
+                try:
+                    header_location = header[curr_cell]
+                except IndexError:
+                    header_location = "no header"
+                location_desc = "Location (SampleID,Header Field)<br>%s,%s" %\
+                 (sample_id_name, header_location)
+            if not all_errs_warnings:
+                formatted_data += "<th><tt>%s</tt></th>" %\
+                 mapping_data[curr_row][curr_cell]
+            elif len(mapping_data[curr_row][curr_cell].replace('\n', '')) == 0:
+                formatted_data += """<th bgcolor=%s><a href="javascript:void(0);" onmouseover="return overlib('%s');" onmouseout="return nd();"><font color=%s><tt>%s</tt></a></th>""" % (bg_color, all_errs_warnings.replace('"', '').replace("'", "") + location_desc, font_color, "missing data")
+            else:
+                formatted_data += """<th bgcolor=%s><a href="javascript:void(0);" onmouseover="return overlib('%s');" onmouseout="return nd();"><font color=%s><tt>%s</tt></a></th>""" % (bg_color, all_errs_warnings.replace('"', '').replace("'", "") + location_desc, font_color, mapping_data[curr_row][curr_cell])
+
+    
+        formatted_data += "</tr>"
+    html_lines += HTML_LINES_DATA % formatted_data
+    
+    return html_lines
+    
+HTML_LINES_INIT = """<html>
+<head>
+
+<script type="text/javascript" src="./overlib.js"></script>
+</head>
+<body bgcolor="white"> """
+
+HTML_LINES_MSG = """<h1>Mapping file error and warning details.</h1>
+Notes for interpreting this report:
+<ul>
+    <li>Errors will be listed in red, warnings in yellow.  
+    <li>Mouse over an error or warning in a cell for more details.
+    <li>Errors in the header row may mask other errors, so these should be corrected first.
+    <li>Modifications to your mapping file to fix certain issues may result in different errors. You should run <tt>check_id_map.py</tt> until no errors (nor warnings, ideally) are found.
+</ul>
+<p>
+Some general rules about formatting mapping files (see <a href="http://qiime.org/documentation/file_formats.html#metadata-mapping-files">here</a> for additional details):
+<ul> 
+    <li>Header characters should only contain alphanumeric and <tt>_</tt> characters only.
+    <li>Valid characters for SampleID fields are alphanumeric and <tt>.</tt> only.<br>
+    <li>Other fields allow alphanumeric and <tt>%s</tt> characters.
+</ul>
+General issues with your mapping file (i.e., those that do not pertain to a particular cell) will be listed here, if any:<table border="1" cellspacing="0" cellpadding="7"><tr>%s%s</tr></table><br>"""
+
+
+HTML_LINES_HEADER = """
+<table border="2" cellspacing="0" cellpadding="5">
+
+<tr></tr>
+<tr>
+%s
+</tr>
+"""
+
+HTML_LINES_DATA = """
+<tr>
+%s
+</tr>
+</table>
+
+</body>
+</html>"""
+
 
