@@ -6,7 +6,7 @@ __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Jesse Stombaugh", "Rob Knight", "Micah Hamady", "Dan Knights",\
 "Antonio Gonzalez Pena"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Jesse Stombaugh"
 __email__ = "jesse.stombaugh@colorado.edu"
 __status__ = "Release"
@@ -80,7 +80,7 @@ def make_3d_plots(coord_header, coords, pct_var, mapping, prefs, \
                     ellipsoid_prefs=None, \
                     user_supplied_edges=False, ball_scale=1.0, \
                     arrow_colors={'line_color': 'white', 'head_color': 'red'},
-                    add_vectors=None):
+                    add_vectors=None,plot_scaled=False,plot_unscaled=True):
     """Makes 3d plots given coords, mapping file, and prefs.
     
     Added quick-and-dirty hack for gradient coloring of columns, should
@@ -142,24 +142,26 @@ def make_3d_plots(coord_header, coords, pct_var, mapping, prefs, \
         data_colors=groups_and_colors[i][3]
         data_color_order=groups_and_colors[i][4]
         
-        result.extend(make_mage_output(groups, colors, coord_header, coords, \
-            pct_var,background_color,label_color,data_colors, \
-            taxa, custom_axes,name=labelname, \
-            scaled=False, edges=edges,
-            coords_low=coords_low, coords_high=coords_high, \
-            ellipsoid_prefs=ellipsoid_prefs, \
-            user_supplied_edges=user_supplied_edges, \
-            ball_scale=ball_scale, arrow_colors=arrow_colors, \
-            add_vectors=add_vectors))
-        result.extend(make_mage_output(groups, colors, coord_header, coords, \
-            pct_var,background_color,label_color,data_colors, \
-            taxa, custom_axes,name=labelname, \
-            scaled=True, edges=edges, \
-            coords_low=coords_low, coords_high=coords_high, \
-            ellipsoid_prefs=ellipsoid_prefs, \
-            user_supplied_edges=user_supplied_edges, \
-            ball_scale=ball_scale, arrow_colors=arrow_colors, \
-            add_vectors=add_vectors))
+        if plot_unscaled:
+            result.extend(make_mage_output(groups, colors, coord_header, coords, \
+                pct_var,background_color,label_color,data_colors, \
+                taxa, custom_axes,name=labelname, \
+                scaled=False, edges=edges,
+                coords_low=coords_low, coords_high=coords_high, \
+                ellipsoid_prefs=ellipsoid_prefs, \
+                user_supplied_edges=user_supplied_edges, \
+                ball_scale=ball_scale, arrow_colors=arrow_colors, \
+                add_vectors=add_vectors))
+        if plot_scaled:
+            result.extend(make_mage_output(groups, colors, coord_header, coords, \
+                pct_var,background_color,label_color,data_colors, \
+                taxa, custom_axes,name=labelname, \
+                scaled=True, edges=edges, \
+                coords_low=coords_low, coords_high=coords_high, \
+                ellipsoid_prefs=ellipsoid_prefs, \
+                user_supplied_edges=user_supplied_edges, \
+                ball_scale=ball_scale, arrow_colors=arrow_colors, \
+                add_vectors=add_vectors))
                 
     return result
 
@@ -283,7 +285,7 @@ master={labels} nobutton' % (color, radius, alpha, num_coords))
             if not scaled and add_vectors['rms_path'] and add_vectors['rms_algorithm']:
                vector_result = make_subgroup_rms(coord_dict, add_vectors['eigvals'], \
                                                  ids, add_vectors['rms_algorithm'], \
-                                                 custom_axes)
+                                                 add_vectors['rms_axes'], custom_axes)
                if not isnan(vector_result['rms']):
                    if name not in add_vectors['rms_output']:
                        add_vectors['rms_output'][name] = {}
@@ -340,7 +342,7 @@ master={labels} nobutton' % (color, radius, alpha, num_coords))
     return result
 
 
-def make_subgroup_rms(coord_dict, eigvals, ids, method='avg', custom_axes=None):
+def make_subgroup_rms(coord_dict, eigvals, ids, method='avg', rms_axes=3, custom_axes=None):
     """Creates the rms value, vector, of a subgroup (ids) of coord_dict
     
        Params: 
@@ -358,21 +360,21 @@ def make_subgroup_rms(coord_dict, eigvals, ids, method='avg', custom_axes=None):
     
     # We multiply the cood values with the value of the eigvals represented
     if custom_axes:
-        vectors = [coord_dict[id][1:]*eigvals for id in ids if id in coord_dict]
+        vectors = [coord_dict[id][1:][:rms_axes]*eigvals[:rms_axes] for id in ids if id in coord_dict]
     else:
-        vectors = [coord_dict[id]*eigvals for id in ids if id in coord_dict]
+        vectors = [coord_dict[id][:rms_axes]*eigvals[:rms_axes] for id in ids if id in coord_dict]
     
     if method=='avg':
        center = average(vectors,axis=0)
        if len(ids)==1:
-           result['vector'] = norm(center)
+           result['vector'] = [norm(center)]
            result['rms'] = result['vector']
        else:
            result['vector'] = [norm(i) for i in vectors-center]
            result['rms'] = average(result['vector'])
     elif method=='trajectory':
        if len(ids)==1:
-           result['vector'] = norm(vectors)
+           result['vector'] = [norm(vectors)]
            result['rms'] = result['vector']
        else:
            result['vector'] = [norm(vectors[i-1]-vectors[i])  for i in range(len(vectors)-1)] 
@@ -387,6 +389,8 @@ def make_subgroup_rms(coord_dict, eigvals, ids, method='avg', custom_axes=None):
 
 def run_ANOVA_trajetories(groups):
     """Run ANOVA on trajectories categories
+    
+       Added some lines to control for labels and make everything a list vs. list/sets
 
        Params: 
         groups, a dict of {trajectory_id: values}, where trajectory_id is
@@ -399,13 +403,14 @@ def run_ANOVA_trajetories(groups):
         This was taken from qiime/otu_category_significance.py:run_single_ANOVA
     """
     values = []
+    labels = []
     for trajectory in groups:
+        labels.append(trajectory)
         values.append(Numbers(groups[trajectory]))
             
     # This is copied from otu_category_significance.py
     try:
         dfn, dfd, F, between_MS, within_MS, group_means, prob = ANOVA_one_way(values)
-        return group_means, prob
     except ValueError:
         #set the p-value to 'diff' if the variances are 0.0 (within rounding 
         #error) and the means are not all the same. If the means are all
@@ -415,12 +420,14 @@ def run_ANOVA_trajetories(groups):
         for i in values:
             group_means.append(i.Mean)
             group_variances.append(i.Variance)
-        group_means = set(group_means)
+        ##### Added list to always return the same type of output
+        group_means = list(set(group_means))
         if sum(group_variances) < 1e-21 and len(group_means) > 1:
             prob = 0.0
         else:
             prob = 1.0
-        return group_means, prob
+    
+    return labels, group_means, prob
         
 def make_vectors_output(coord_dict, add_vectors, num_coords, color, ids):
     """Creates make output to display vectors (as a kinemage vectorlist).
@@ -954,7 +961,7 @@ def generate_3d_plots(prefs, data, custom_axes, background_color, label_color, \
                         default_filename='out', ellipsoid_prefs=None, \
                         user_supplied_edges=False, ball_scale=1.0, \
                         arrow_colors={'line_color': 'white', 'head_color': 'red'}, \
-                        add_vectors=None):
+                        add_vectors=None,plot_scaled=False,plot_unscaled=True):
     """Make 3d plots according to coloring options in prefs."""
     
     if filename is None:
@@ -983,7 +990,8 @@ def generate_3d_plots(prefs, data, custom_axes, background_color, label_color, \
                         ellipsoid_prefs=ellipsoid_prefs, \
                         user_supplied_edges=user_supplied_edges, \
                         ball_scale=ball_scale, arrow_colors=arrow_colors,
-                        add_vectors=add_vectors)
+                        add_vectors=add_vectors, plot_scaled=plot_scaled, \
+                        plot_unscaled=plot_unscaled)
     
     
     # Validating if we should add RMS values
@@ -993,17 +1001,25 @@ def generate_3d_plots(prefs, data, custom_axes, background_color, label_color, \
         f_rms.write('Method to calculate the RMS: %s\n\n' % add_vectors['rms_algorithm'])
         
         for group in add_vectors['rms_output']:
-            to_test = {}
-            for cat in add_vectors['rms_output'][group]:
-                if len(add_vectors['rms_output'][group][cat]['rms_vector']) != 0:
-                    to_test[cat] = add_vectors['rms_output'][group][cat]['rms_vector']
+            if len(add_vectors['rms_output'][group].keys())==1:
+                f_rms.write('Grouped by %s: Only one value in the group.\n\n' \
+                    % (group))
+            else:
+                to_test = {}
+                categories_values = []
+                for cat in add_vectors['rms_output'][group]:
+                    if len(add_vectors['rms_output'][group][cat]['rms_vector']) != 0:
+                        to_test[cat] = add_vectors['rms_output'][group][cat]['rms_vector']
+                    else:
+                        add_vectors['rms_output'][group][cat]['rms_result'] = nan
+                
+                labels, group_means, prob = run_ANOVA_trajetories(to_test)
+                if (len(labels)==len(group_means)):
+                    f_rms.write('Grouped by %s, probability: %f\nGroup means (ANOVA): %s\n\n' \
+                        % (group, prob, [(labels[i], group_means[i]) for i in range(len(labels))]))
                 else:
-                    add_vectors['rms_output'][group][cat]['rms_result'] = nan
-            
-            group_means, prob = run_ANOVA_trajetories(to_test)
-            f_rms.write('Grouped by %s, probability: %f\nGroup means (ANOVA): %s\n\n' \
-                % (group, prob, group_means))
-            
+                    f_rms.write('Grouped by %s: this value can not be used\n\n' % (group))
+                    
         f_rms.close()
     
     #Write kinemage file
