@@ -6,11 +6,12 @@ __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Rob Knight", "Daniel McDonald", "Greg Caporaso",
     "Justin Kuczynski", "Cathy Lozupone", "Jens Reeder", "Antonio Gonzalez Pena"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
 
+from biom.table import table_factory
 from string import strip
 from collections import defaultdict
 from cogent.util.dict2d import Dict2D
@@ -391,6 +392,32 @@ def parse_taxonomy(infile):
 
     return res
 
+def taxa_split(taxa_string):
+    return [t.strip() for t in taxa_string.split(';')]
+    
+def parse_taxonomy_to_otu_metadata(lines,labels=['taxonomy','score'],process_fs=[taxa_split,float]):
+    """ """
+    result = {}
+    for line in lines:
+        line = line.strip()
+        fields = line.split('\t')
+        id_ = fields[0].split()[0]
+        result[id_] = {}
+        for i,field in enumerate(fields[1:]):
+            try:
+                value = process_fs[i](field)
+            except IndexError:
+                raise ValueError, "Too few process functions provided (n=%d), or too many metadata entries for OTU %s."\
+                  % (len(process_fs),id_)
+            try:
+                label = labels[i]
+            except IndexError:
+                raise ValueError, "Too few labels provided (n=%d), or too many metadata entries for OTU %s."\
+                  % (len(labels),id_)
+            
+            result[id_][label] = value
+    return result
+
 def process_otu_table_sample_ids(sample_id_fields):
     """ process the sample IDs line of an OTU table """
     if len(sample_id_fields) == 0:
@@ -413,8 +440,8 @@ def process_otu_table_sample_ids(sample_id_fields):
     # column is included.
     return sample_ids, has_metadata
 
-def parse_otu_table(lines,count_map_f=int):
-    """parses otu table (sample ID x OTU ID map)
+def parse_classic_otu_table(lines,count_map_f=int):
+    """parses a classic otu table (sample ID x OTU ID map)
 
     Returns tuple: sample_ids, otu_ids, matrix of OTUs(rows) x samples(cols),
     and lineages from infile.
@@ -476,9 +503,11 @@ def parse_otu_table(lines,count_map_f=int):
                             otu_table.append(array(map(float, fields[1:])))
                         
     return sample_ids, otu_ids, array(otu_table), metadata
+parse_otu_table = parse_classic_otu_table
+
 
 def parse_taxa_summary_table(lines):
-    result = parse_otu_table(lines,count_map_f=float)
+    result = parse_classic_otu_table(lines,count_map_f=float)
     return result[0], result[1], result[2]
 
 def filter_otus_by_lineage(sample_ids, otu_ids, otu_table, lineages, \
@@ -593,6 +622,36 @@ def sample_mapping_to_otu_table(lines):
         out.append('\t'.join(new_line))
     return out
 
+def sample_mapping_to_biom_table(lines):
+    """Converts the UniFrac sample mapping file to biom table object
+    
+    The sample mapping file is a required input for the UniFrac web interface.
+    """
+    data = []
+    sample_ids = []
+    observation_ids = []
+    for line in lines:
+        fields = line.strip().split()
+        observation_id = fields[0]
+        sample_id = fields[1]
+        count = float(fields[2])
+        
+        try:
+            sample_idx = sample_ids.index(sample_id)
+        except ValueError:
+            sample_idx = len(sample_ids)
+            sample_ids.append(sample_id)
+        try:
+            observation_idx = observation_ids.index(observation_id)
+        except ValueError:
+            observation_idx = len(observation_ids)
+            observation_ids.append(observation_id)
+            
+        data.append([observation_idx, sample_idx, count])
+    
+    return table_factory(data,sample_ids,observation_ids)
+
+
 def parse_sample_mapping(lines):
     """Parses the UniFrac sample mapping file (environment file)
 
@@ -831,3 +890,41 @@ def parse_denoiser_mapping(denoiser_map):
         else:
             result[denoised_id] = original_ids
     return result
+    
+def parse_otu_map(otu_map_f,otu_ids_to_exclude=None,delim='_'):
+    """ parse otu map file into a sparse dict {(otu_idx,sample_idx):count}
+    
+        This function is much more memory efficent than fields_to_dict and
+         and the result dict is of the correct format to be passed to 
+         table_factory for creating OtuTable objects.
+    
+    """
+    if otu_ids_to_exclude == None:
+        otu_ids_to_exclude = {}
+    
+    result = defaultdict(int)
+    sample_ids = []
+    sample_id_idx = {}
+    otu_ids = []
+    otu_count = 0
+    sample_count = 0
+    for line in otu_map_f:
+        fields = line.strip().split('\t')
+        otu_id = fields[0]
+        if otu_id in otu_ids_to_exclude:
+            continue
+        for seq_id in fields[1:]:
+            sample_id = seq_id.split(delim)[0]
+            try:
+                sample_index = sample_id_idx[sample_id]
+            except KeyError:
+                sample_index = sample_count
+                sample_id_idx[sample_id] = sample_index
+                sample_count += 1
+                sample_ids.append(sample_id)
+            # {(row,col):val}
+            result[(otu_count,sample_index)] += 1
+        otu_count += 1
+        otu_ids.append(otu_id)
+    return result, sample_ids, otu_ids
+    
