@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 __author__ = "Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME Project" 
-__credits__ = ["Rob Knight","Greg Caporaso", "Kyle Bittinger","Jens Reeder","William Walters"]
+__credits__ = ["Rob Knight","Greg Caporaso", "Kyle Bittinger","Jens Reeder", "William Walters", "Jose Carlos Clemente Litran"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -23,7 +23,7 @@ from cogent.parse.fasta import MinimalFastaParser
 from cogent.parse.mothur import parse_otu_list as mothur_parse
 from cogent.app.cd_hit import cdhit_clusters_from_seqs
 from cogent.app.dotur import dotur_from_alignment
-from cogent.app.mothur import Mothur
+from qiime.pycogent_backports.mothur import Mothur
 from cogent.app.formatdb import build_blast_db_from_fasta_path
 from cogent.app.blast import blast_seqs, Blastall, BlastResult
 from cogent.core.sequence import DnaSequence
@@ -35,7 +35,7 @@ from qiime.util import FunctionWithParams, get_tmp_filename
 from qiime.sort import sort_fasta_by_abundance
 from qiime.parse import fields_to_dict
 from qiime.pycogent_backports.uclust import get_clusters_from_fasta_filepath
-from qiime.pycogent_backports.usearch import otu_pipe
+from qiime.pycogent_backports.usearch import usearch_qf
 
 class OtuPicker(FunctionWithParams):
     """An OtuPicker dereplicates a set of sequences at a given similarity.
@@ -314,7 +314,7 @@ class BlastOtuPicker(OtuPicker):
                 # if the alignment is the longest we've seen so far (or 
                 # the first), hold on to it as a possible best hit
                 len_current = blast_hit['ALIGNMENT LENGTH']
-                if len_current >= len_longest:
+                if len_current > len_longest:
                     choice = blast_hit
                     len_longest = len_current 
             query = query.split()[0]    #get rid of spaces
@@ -696,7 +696,8 @@ class UclustOtuPickerBase(OtuPicker):
     def _presort_by_abundance(self,seq_path):
         """ Preform pre-sorting of input by abundance """
         
-        # Turn off uclust's sorting
+        # Turn off uclust's sorting - if doing our presort by
+        # abundance we _always_ need to disable uclust's sorting.
         self.Params['suppress_sort'] = True
         
         # Get a temp file name for the sorted fasta file
@@ -844,6 +845,7 @@ class UclustOtuPicker(UclustOtuPickerBase):
          output_dir=self.Params['output_dir'],
          HALT_EXEC=HALT_EXEC)
         
+        
         # clean up any temp files that were created
         remove_files(self.files_to_remove)
         
@@ -917,6 +919,7 @@ class UsearchOtuPicker(UclustOtuPickerBase):
          'cluster_size_filtering':True,
          'output_dir':'.',
          'remove_usearch_logs':False,
+         'derep_fullseq':False,
          'chimeras_retention':'union',
          'verbose':False}
          
@@ -948,7 +951,7 @@ class UsearchOtuPicker(UclustOtuPickerBase):
         
         
         # perform the filtering/clustering
-        clusters, failures = otu_pipe(
+        clusters, failures = usearch_qf(
          seq_path,
          output_dir = self.Params['output_dir'],
          percent_id = self.Params['percent_id'],
@@ -974,6 +977,7 @@ class UsearchOtuPicker(UclustOtuPickerBase):
          reference_chimera_detection=self.Params['reference_chimera_detection'],
          cluster_size_filtering = self.Params['cluster_size_filtering'],
          remove_usearch_logs = self.Params['remove_usearch_logs'],
+         derep_fullseq = self.Params['derep_fullseq'],
          chimeras_retention = self.Params['chimeras_retention'],
          verbose = self.Params['verbose'],
          HALT_EXEC=HALT_EXEC)
@@ -1059,6 +1063,7 @@ class UsearchReferenceOtuPicker(UclustOtuPickerBase):
          'output_dir':'.',
          'remove_usearch_logs':False,
          'suppress_new_clusters':False,
+         'derep_fullseq':False,
          'chimeras_retention':'union',
          'verbose':False}
          
@@ -1091,7 +1096,7 @@ class UsearchReferenceOtuPicker(UclustOtuPickerBase):
         
         
         # perform the filtering/clustering
-        clusters, failures = otu_pipe(
+        clusters, failures = usearch_qf(
          seq_path,
          refseqs_fp,
          output_dir = self.Params['output_dir'],
@@ -1119,6 +1124,7 @@ class UsearchReferenceOtuPicker(UclustOtuPickerBase):
          cluster_size_filtering = self.Params['cluster_size_filtering'],
          remove_usearch_logs = self.Params['remove_usearch_logs'],
          suppress_new_clusters = self.Params['suppress_new_clusters'],
+         derep_fullseq = self.Params['derep_fullseq'],
          chimeras_retention = self.Params['chimeras_retention'],
          verbose = self.Params['verbose'],
          HALT_EXEC=HALT_EXEC)
