@@ -4,13 +4,13 @@ import os.path
 from optparse import OptionParser
 from cogent.maths.fit_function import fit_function
 from qiime.parse import parse_distmat
-from numpy import exp, cos, pi, tri, argsort, asarray, arange, mean, isnan, zeros
+from numpy import exp, cos, pi, tri, argsort, asarray, arange, mean, isnan, zeros, square
 
 __author__ = "Antonio Gonzalez Pena"
 __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Antonio Gonzalez Pena"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Antonio Gonzalez Pena"
 __email__ = "antgonza@gmail.com"
 __status__ = "Release"
@@ -23,6 +23,7 @@ class FitModel(object):
     def __init__(self, x, y, model):
         self.x = x
         self.y = y
+        self.model_text = model
         self.model = self._get_model(model)
 
     # Funcion definition -- defining these in your function makes this 
@@ -30,19 +31,24 @@ class FitModel(object):
     options = ['nugget','exponential','gaussian','periodic', 'linear']
     
     def _linear(self, x, a):
+        self.text = "%f+(%f*x)" % (a[0], a[1])
         return a[0]+(a[1]*x)
     
     def _periodic(self, x, a):
-        return a[0]+(a[2]*(1-cos(2*pi*x/a[1])))
+        self.text = "%f+((1-cos(2*pi*x/%f))*%f)" % (a[0], a[1], a[2])
+        return a[0]+((1-cos(2*pi*x/a[1]))*a[2])
     
     def _gaussian(self, x, a):
-        return a[0]+(a[2]*(1-exp((-3*x*x)/(a[1]*a[1]))))
+        self.text = "%f+((1-exp((-3*x*x)/square(%f)))*%f)" % (a[0], a[1], a[2])
+        return a[0]+((1-exp((-3*x*x)/square(a[1])))*a[2])
     
     def _exponential(self, x, a):
-        return a[0]+(a[2]*(1-exp(-3*x/a[1])))
+        self.text = "%f+((1-exp(-3*x/%f))*%f)" % (a[0], a[1], a[2])
+        return a[0]+((1-exp(-3*x/a[1]))*a[2])
         
     def _nugget(self, x, a):
-        return a[0] 
+        self.text = "%f" % (a[0])
+        return a[0]
 
     def _get_model(self, model):
         if model == 'linear':
@@ -59,14 +65,16 @@ class FitModel(object):
             raise ValueError, "Unknown model type: %s" % model
     
     def __call__(self):
-        if self.model!='nugget':
+        if self.model_text!='nugget':
             # what are 3 and 10? should these be parametrizable?
             params = fit_function(self.x, self.y, self.model, 3, 10)
+            y = self.model(self.x, params)
         else:
             # what are 1 and 1? should these be parametrizable?
             params = fit_function(self.x, self.y, self.model, 1, 1)
-        
-        return self.model(self.x, params)
+            y = [self.model(self.x, params)] * len(self.x)
+            
+        return y, params, self.text
         
 def hist_bins(bins, vals):
     """ Creates a histogram given the bins and the vals
@@ -138,7 +146,7 @@ def fit_semivariogram((x_samples,x_distmtx), (y_samples,y_distmtx), model, range
     
     # fitting model
     fit_func = FitModel(x_vals, y_vals, model)
-    y_fit = fit_func()
+    y_fit, params, func_text = fit_func()
     x_fit = x_vals
     
     # section for bins
@@ -171,5 +179,5 @@ def fit_semivariogram((x_samples,x_distmtx), (y_samples,y_distmtx), model, range
         x_vals = x_vals[~isnan(y_vals)]
         y_vals = y_vals[~isnan(y_vals)]
     
-    return x_vals, y_vals, x_fit, y_fit
+    return x_vals, y_vals, x_fit, y_fit, func_text
     
\ No newline at end of file
