@@ -7,15 +7,15 @@ __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Jesse Stobmaugh","Julia Goodrich",\
                "Micah Hamady"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.3.0"
+__version__ = "1.5.0"
 __maintainer__ = "Jesse Stombaugh"
 __email__ = "jesse.stombaugh@colorado.edu"
 __status__ = "Release"
 
 """
 Requirements:
-MatPlotLib 0.98.5.2
-Python 2.5
+MatPlotLib 1.1.0
+Python 2.7
 """
 
 import matplotlib,re
