@@ -6,7 +6,7 @@ __author__ = "Justin Kuczynski"
 __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Justin Kuczynski", "Rob Knight"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Justin Kuczynski"
 __email__ = "justinak@gmail.com"
 __status__ = "Release"
@@ -16,12 +16,10 @@ import random
 
 from qiime.util import make_option
 
-from qiime.parse import parse_otu_table
 from qiime.util import parse_command_line_parameters
-from qiime.format import format_otu_table
 from qiime.sort import natsort
 
-def sim_otu_table(sample_ids, otu_ids, otu_mtx, otu_metadata, tree, 
+def sim_otu_table(sample_ids, otu_ids, samples, otu_metadata, tree, 
     num_replicates, dissimilarity):
     """ make n samples related to each sample in an input otu table
 
@@ -29,7 +27,8 @@ def sim_otu_table(sample_ids, otu_ids, otu_mtx, otu_metadata, tree,
     the constituents of an otu table
      * sample_ids: list
      * otu_ids: list
-     * otu_mtx: numpy2d otus as rows
+     * samples: iterable, each element must have sample_vector = elem[0]
+       where sample_vector looks like [1,0,7,9...]
      * otu_metadata: list, either empty or of length len(otu_ids)
     tree: a PhyloNode tree or compatible,
     num_replicates: how many replicates for each input sample
@@ -47,8 +46,9 @@ def sim_otu_table(sample_ids, otu_ids, otu_mtx, otu_metadata, tree,
     # format of sample_dict: otu_id: num_seqs
     sample_dicts = []
     res_sam_names = []
-    for i in range(len(sample_ids)):
-        sample_vector = otu_mtx[:,i]
+    for i, sample_info in enumerate(samples):
+        sample_vector = sample_info[0]
+        # sample_vector = otu_observations.next()
         for j in range(num_replicates):
             sample_dict = {}
             for k in range(len(otu_ids)):
@@ -64,6 +64,7 @@ def sim_otu_table(sample_ids, otu_ids, otu_mtx, otu_metadata, tree,
             sample_dicts.append(sample_dict)
             res_sam_names.append(sample_ids[i] + '.' + str(j))
 
+
     res_otu_mtx, res_otus = combine_sample_dicts(sample_dicts)
 
     res_otu_metadata = []
