@@ -6,7 +6,7 @@ __author__ = "Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Greg Caporaso"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -21,12 +21,16 @@ from qiime.format import (format_histogram_one_count,
                           format_split_libraries_fastq_log)
 from qiime.hamming import decode_hamming_8
 from qiime.golay import decode_golay_12
+from qiime.quality import phred_to_ascii33, phred_to_ascii64
 
 class FastqParseError(Exception):
     pass
 
 def get_illumina_qual_chars():
-    return '@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~'
+    # pulled from stack overflow (url wrapped over two lines):
+    # http://stackoverflow.com/questions/5891453/
+    # is-there-a-python-library-that-contains-a-list-of-all-the-ascii-characters
+    return [chr(i) for i in range(127)]
 
 def bad_chars_from_threshold(first_bad_char):
     if first_bad_char == '':
@@ -93,7 +97,7 @@ def quality_filter_sequence(header,
     else:
         return 0, sequence, quality
 
-def check_header_match(header1,header2):
+def check_header_match_pre180(header1,header2):
     
     # split on '#' and '/' to handle cases with and without the
     # Illumina quality digit
@@ -101,6 +105,19 @@ def check_header_match(header1,header2):
     header2 = header2.split('#')[0].split('/')[0]
     
     return header1 == header2
+
+def check_header_match_180_or_later(header1,header2):
+    """ Confirm headers are compatible in CASAVA 1.8.0 or later
+        
+        These contain information on the read number, so can differ
+    """
+    header1 = header1.split(':')
+    header2 = header2.split(':')
+    for e1,e2 in zip(header1,header2):
+        if e1.split(' ')[0] != e2.split(' ')[0]:
+            return False
+    
+    return True
     
 BARCODE_DECODER_LOOKUP = {
  'golay_12':decode_golay_12,
@@ -137,19 +154,28 @@ def correct_barcode(barcode,barcode_to_sample_id,correction_fn):
         
         return num_errors, corrected_barcode, True, sample_id
 
+def is_casava_v180_or_later(header_line):
+    """ True if this file is generated by Illumina software post-casava 1.8 """
+    assert header_line.startswith('@'),\
+     "Non-header line passed as input. Header must start with '@'."
+    fields = header_line.split(':')
+    if len(fields) == 10 and fields[7] in 'YN':
+        return True
+    return False
+
 
 def process_fastq_single_end_read_file(fastq_read_f,
                                        fastq_barcode_f,
                                        barcode_to_sample_id,
                                        store_unassigned=False,
                                        max_bad_run_length=0,
-                                       last_bad_quality_char='B',
-                                       min_per_read_length=75,
+                                       phred_quality_threshold=2,
+                                       min_per_read_length_fraction=0.75,
                                        rev_comp=False,
                                        rev_comp_barcode=False,
                                        seq_max_N=0,
                                        start_seq_id=0,
-                                       filter_bad_illumina_qual_digit=True,
+                                       filter_bad_illumina_qual_digit=False,
                                        log_f=None,
                                        histogram_f=None,
                                        barcode_correction_fn=None,
@@ -162,6 +188,41 @@ def process_fastq_single_end_read_file(fastq_read_f,
     
     seq_id = start_seq_id
     
+    # grab the first lines and then seek back to the beginning of the file
+    try:
+        fastq_read_f_line1 = fastq_read_f.readline()
+        fastq_read_f_line2 = fastq_read_f.readline()
+        fastq_read_f.seek(0)
+    except AttributeError:
+        fastq_read_f_line1 = fastq_read_f[0]
+        fastq_read_f_line2 = fastq_read_f[1]
+    
+    post_casava_v180 = is_casava_v180_or_later(fastq_read_f_line1)
+    if post_casava_v180:
+        phred_to_ascii_f = phred_to_ascii33
+        check_header_match_f = check_header_match_180_or_later
+    else:
+        phred_to_ascii_f = phred_to_ascii64
+        check_header_match_f = check_header_match_pre180
+    
+    if phred_quality_threshold != None:
+        last_bad_quality_char = phred_to_ascii_f(phred_quality_threshold)
+    else:
+        # disable quality filter
+        last_bad_quality_char = ''
+    
+    # compute the barcode length, if they are all the same. 
+    # this is useful for selecting a subset of the barcode read
+    # if it's too long (e.g., for technical reasons on the sequencer)
+    barcode_lengths = set([len(bc) for bc, sid in barcode_to_sample_id.items()])
+    if len(barcode_lengths) == 1:
+        barcode_length = barcode_lengths.pop()
+    else:
+        barcode_length = None
+    
+    # compute the minimum read length as a fraction of the length of the input read
+    min_per_read_length = min_per_read_length_fraction * len(fastq_read_f_line2)
+    
     # prep data for logging
     input_sequence_count = 0
     count_barcode_not_in_map = 0
@@ -176,8 +237,8 @@ def process_fastq_single_end_read_file(fastq_read_f,
                                   MinimalFastqParser(fastq_read_f,strict=False)):
         input_sequence_count += 1
         # Confirm match between barcode and read headers
-        if not check_header_match(bc_data[header_index],
-                                  read_data[header_index]):
+        if not check_header_match_f(bc_data[header_index],
+                                    read_data[header_index]):
             raise FastqParseError,\
              ("Headers of barcode and read do not match. Can't continue. "
               "Confirm that the barcode fastq and read fastq that you are "
@@ -186,7 +247,16 @@ def process_fastq_single_end_read_file(fastq_read_f,
             header = read_data[header_index]
         
         # Grab the barcode sequence
-        barcode = bc_data[sequence_index]
+        if barcode_length:
+            # because thirteen cycles are sometimes used for 
+            # techical reasons, this step looks only at the 
+            # first tweleve bases. note that the barcode is
+            # rev-comp'ed after this step if requested since
+            # the thirteen base is a technical artefact, not
+            # barcode sequence.
+            barcode = bc_data[sequence_index][:barcode_length]
+        else:
+            barcode = bc_data[sequence_index]
         if rev_comp_barcode:
             barcode = DNA.rc(barcode)
         # Grab the read sequence
@@ -250,6 +320,12 @@ def process_fastq_single_end_read_file(fastq_read_f,
           (sample_id,seq_id,header,barcode,corrected_barcode,num_barcode_errors)
         yield fasta_header, sequence, quality, seq_id
         seq_id += 1
+    
+    # Add sample IDs with zero counts to dictionary for logging    
+    for curr_sample_id in barcode_to_sample_id.values():
+        if curr_sample_id not in seqs_per_sample_counts.keys():
+            seqs_per_sample_counts[curr_sample_id] = 0
+    
 
     if log_f != None:
         log_str = format_split_libraries_fastq_log(count_barcode_not_in_map,
