@@ -3,9 +3,11 @@
 __author__ = "Daniel McDonald"
 __copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Rob Knight", "Daniel McDonald", "Greg Caporaso", 
-"Justin Kuczynski","Jens Reeder","Catherine Lozupone"] #remember to add yourself if you make changes
+               "Justin Kuczynski", "Jens Reeder", "Catherine Lozupone",
+               "Jai Ram Rideout", "Logan Knecht", "Michael Dwan",
+               "Levi McCracken", "Damien Coy"] #remember to add yourself if you make changes
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -20,7 +22,7 @@ from StringIO import StringIO
 from os import getenv, makedirs
 from operator import itemgetter
 from os.path import abspath, exists, dirname, join, isdir
-from collections import defaultdict
+from collections import defaultdict, Counter
 import gzip
 import sys
 import os
@@ -35,6 +37,8 @@ from numpy.ma import MaskedArray
 from numpy.ma.extras import apply_along_axis
 from numpy import array, zeros, argsort, shape, vstack,ndarray, asarray, \
         float, where, isnan, mean, std, sqrt, ravel
+
+from biom.table import DenseTable
         
 from cogent.util.dict2d import Dict2D
 from cogent import LoadSeqs, Sequence,DNA
@@ -45,7 +49,8 @@ from cogent.core.moltype import MolType, IUPAC_DNA_chars, IUPAC_DNA_ambiguities,
 from cogent.data.molecular_weight import DnaMW
 from cogent.core.sequence import DnaSequence
 from cogent.app.blast import Blastall
-from cogent.app.util import get_tmp_filename as cogent_get_tmp_filename, FilePath
+from cogent.app.util import (ApplicationError, CommandLineApplication,
+    get_tmp_filename as cogent_get_tmp_filename, FilePath)
 from cogent.parse.blast import BlastResult
 from cogent.parse.fasta import MinimalFastaParser
 from cogent.util.misc import remove_files
@@ -58,7 +63,8 @@ from cogent.util.misc import (create_dir,
 from qiime import __version__ as qiime_library_version
 from cogent.util.option_parsing import (parse_command_line_parameters,
                                         make_option)
-from qiime.parse import (parse_otu_table,
+from qiime.parse import (parse_distmat,
+                         parse_mapping_file_to_dict,
                          parse_qiime_config_files,
                          parse_coords,
                          parse_newick,
@@ -67,7 +73,7 @@ from qiime.parse import (parse_otu_table,
                          parse_mapping_file,
                          parse_denoiser_mapping,
                          MinimalFastqParser)
-from qiime.format import format_otu_table
+
 
 class TreeMissingError(IOError):
     """Exception for missing tree file"""
@@ -158,30 +164,6 @@ class FunctionWithParams(object):
         f.write(self.formatResult(result))
         f.close()
 
-    def getOtuTable(self, otu_source):
-        """Returns parsed OTU table from putative OTU source."""
-        
-        #if we have a string starting with #, assume it's an OTU file,
-        #otherwise assume it's a path
-        # if 4-tuple, just return it
-        if type(otu_source) == type((1,3,4,44)):
-            return otu_source
-        if hasattr(otu_source, 'startswith') and otu_source.startswith('#'):
-            try:
-                return parse_otu_table(StringIO(otu_source))
-            except (TypeError, ValueError), e:
-                raise OtuMissingError, \
-                    "Tried to read OTUs from string starting with # but got "+e
-        else:
-            try:
-                otu_file = open(otu_source, 'U')
-            except (TypeError, IOError):
-                raise OtuMissingError, \
-                    "Couldn't read OTU file at path: %s" % otu_source
-            result = parse_otu_table(otu_file)
-            otu_file.close()
-            return result
-
     def getTree(self, tree_source):
         """Returns parsed tree from putative tree source"""
         if isinstance(tree_source, PhyloNode):
@@ -250,6 +232,12 @@ class FunctionWithParams(object):
         else:
             return result
 
+def trim_fastq(fastq_lines,output_length):
+    """trim fastq seqs/quals to output_length bases """
+    for seq_id, seq, qual in MinimalFastqParser(fastq_lines,strict=False):
+        yield '@%s\n%s\n+\n%s\n' % (seq_id,seq[:output_length],
+                                      qual[:output_length])
+
 def get_qiime_project_dir():
     """ Returns the top-level QIIME directory
     """
@@ -484,14 +472,29 @@ def split_fasta_on_sample_ids_to_files(seqs,
         write_seqs_to_fasta(current_fp,current_seqs,write_mode='a')
     return None
 
-def compute_seqs_per_library_stats(otu_f):
-    counts = []
-    sample_ids, otu_ids, otu_table, lineages = parse_otu_table(otu_f)
-    for i in range(otu_table.shape[1]):
-        counts.append(sum(otu_table[:,i]))
-        
-    return min(counts), max(counts), median(counts), mean(counts),\
-     dict(zip(sample_ids,counts))
+def compute_seqs_per_library_stats(otu_table, otu_counts=False):
+    """Return summary statistics on per-sample observation counts
+    
+        otu_table: an OTUTable object
+    
+    """
+    if otu_counts:
+        sample_counts = {}
+        for count_vector, sample_id, metadata in otu_table.iterSamples():
+            sample_counts[sample_id] = len([x for x in count_vector if x != 0])
+        counts = sample_counts.values()
+    else:
+        sample_counts = {}
+        for count_vector, sample_id, metadata in otu_table.iterSamples():
+            sample_counts[sample_id] = count_vector.sum()
+        counts = sample_counts.values()
+    
+    return (min(counts),
+            max(counts),
+            median(counts),
+            mean(counts),
+            sample_counts)
+     
 
 def median_absolute_deviation(x):
     """ compute the median of the absolute deviations from the median """
@@ -560,6 +563,9 @@ def get_options_lookup():
     result['output_fp'] =\
      make_option('-o','--output_fp',type="new_filepath",
      help='the output filepath')
+    result['output_biom_fp'] =\
+     make_option('-o','--output_biom_fp',type="new_filepath",
+     help='the output otu table in biom format (recommended extension: .biom)')
     result['mapping_fp'] =\
      make_option('-m','--mapping_fp',type="existing_filepath",
      help='the mapping filepath')
@@ -643,100 +649,6 @@ def matrix_stats(headers_list, distmats):
     
     return deepcopy(headers_list[0]), means, medians, stdevs
 
-
-def merge_otu_tables(otu_table_f1,otu_table_f2):
-    """ Merge two otu tables with non-overlapping sample IDs
-    
-        WARNING: The OTU ids must refer to the same OTUs, which
-         typically only happens when OTUs were picked against a 
-         reference database, as with the BLAST OTU picker.
-    
-    """
-    if isinstance(otu_table_f1, tuple):
-        sample_ids1, otu_ids1, otu_table1, lineages1 = otu_table_f1
-    else:
-        sample_ids1, otu_ids1, otu_table1, lineages1 =\
-            parse_otu_table(otu_table_f1)
-    
-    if isinstance(otu_table_f2, tuple):
-        sample_ids2, otu_ids2, otu_table2, lineages2 = otu_table_f2
-    else:
-        sample_ids2, otu_ids2, otu_table2, lineages2 =\
-            parse_otu_table(otu_table_f2)
-    
-    # assert set(sample_ids1) & set(sample_ids2) == set(),\
-    #  'Overlapping sample ids detected:\n %s' %\
-    #  ' '.join(set(sample_ids1) & set(sample_ids2))
-    sample_ids_result = []
-    sample_ids_result_lookup = {}
-    i = 0
-    for sid in sample_ids1 + sample_ids2:
-        if sid not in sample_ids_result_lookup:
-            sample_ids_result.append(sid)
-            sample_ids_result_lookup[sid] = i
-            i += 1
-        else:
-            pass
-    
-    if lineages1 and lineages2:    
-        # map OTU ids to lineages -- in case of conflicts (i.e, OTU assigned)
-        # different lineage in different otu tables, the lineage from 
-        # OTU table 1 will be taken
-        lineages = True
-        otu_id_to_lineage = dict(zip(otu_ids1,lineages1))
-        otu_id_to_lineage.update(dict([(otu_id,lineage)\
-         for otu_id,lineage in zip(otu_ids2,lineages2)\
-         if otu_id not in otu_id_to_lineage]))
-    elif not (lineages1 or lineages2):
-        lineages = False
-    else:
-      raise ValueError, ('Taxonomic information must be provided either'
-       ' for all or none of the OTU tables')
-    
-    # Get the union of the otu IDs
-    otu_ids_result = list(otu_ids1)
-    otu_ids_lookup = {}.fromkeys(otu_ids1)
-    otu_ids_result.extend([otu_id for otu_id in otu_ids2 \
-                                  if otu_id not in otu_ids_lookup])
-    otu_ids_result_lookup = dict(
-     [(oid,i) for i, oid in enumerate(otu_ids_result)])
-    
-    otu_table = zeros(shape=(len(otu_ids_result),len(sample_ids_result)),dtype=int)
-    for i,sample_id in enumerate(sample_ids1):
-        col_index = sample_ids_result_lookup[sample_id]
-        for j,otu_id in enumerate(otu_ids1):
-            row_index = otu_ids_result_lookup[otu_id]
-            otu_table[row_index,col_index] = otu_table1[j,i]
-        
-    for i,sample_id in enumerate(sample_ids2):
-        col_index = sample_ids_result_lookup[sample_id]
-        for j,otu_id in enumerate(otu_ids2):
-            row_index = otu_ids_result_lookup[otu_id]
-            otu_table[row_index,col_index] += otu_table2[j,i]
-    
-    if lineages:
-        lineages_result = [otu_id_to_lineage[otu_id] 
-         for otu_id in otu_ids_result]
-    else:
-        lineages_result = None
-    
-    return sample_ids_result, otu_ids_result, otu_table, lineages_result
-    
-def merge_n_otu_tables(otu_table_fs):
-    """ Merge n otu tables """
-    if len(otu_table_fs) < 2:
-        raise ValueError, "Two or more OTU tables must be provided."
-    otu_table_f0 = otu_table_fs[0]
-    for otu_table_f in otu_table_fs[1:]:
-        sample_names, otu_names, data, taxonomy = \
-         merge_otu_tables(otu_table_f0,otu_table_f)
-        otu_table_f0 = format_otu_table(sample_names=sample_names, 
-                                    otu_names=otu_names,
-                                    data=data,
-                                    taxonomy=taxonomy).split('\n')
-    
-    return sample_names, otu_names, data, taxonomy
-
 def convert_otu_table_relative(otu_table):
     """Convert the OTU table to relative abundances
 
@@ -1440,4 +1352,396 @@ def summarize_otu_sizes_from_otu_map(otu_map_f):
     return result
 
 
+class DistanceMatrix(DenseTable):
+    """This class represents a QIIME distance matrix.
+    
+    Public attributes:
+        SampleIds - the list of sample ID strings (i.e. row/column headers)
+    """
 
+    _biom_type = "Distance matrix"
+
+    @staticmethod
+    def parseDistanceMatrix(lines):
+        """Parses a QIIME distance matrix file into a DistanceMatrix object.
+        
+        This static method is basically a factory that reads in the given
+        distance matrix file contents and returns a DistanceMatrix instance.
+        This method is provided for convenience.
+
+        Arguments:
+            lines - a list of strings representing the file contents of a QIIME
+                distance matrix
+        """
+        sample_ids, matrix_data = parse_distmat(lines)
+        return DistanceMatrix(matrix_data, sample_ids, sample_ids)
+
+    def __init__(self, *args, **kwargs):
+        """Instantiates a DistanceMatrix object.
+
+        A distance matrix must be square and its sample IDs are exactly the
+        same as its observation IDs (a biom table has sample IDs for column
+        labels and observation IDs for row labels). A distance matrix must be
+        at least 1x1 in size.
+
+        Please refer to the biom.table.Table class documentation for a list of
+        acceptable arguments to the constructor. The data matrix argument (the
+        first argument) is expected to be a numpy array.
+        
+        We have to match the parent class constructor exactly in this case due
+        to how several of the parent class methods are implemented (they assume
+        all subclasses have the same constructor signature). Otherwise, I would
+        have just made a simple constructor that took the matrix data and a
+        single list of sample IDs (because the row/col IDs are the same for a
+        distance matrix). As there is no easy way around this at the moment,
+        users of this class must pass the same list of sample IDs as the
+        observation IDs parameter as well.
+        """
+        super(DistanceMatrix, self).__init__(*args, **kwargs)
+
+        # Make sure the matrix isn't empty, is square, and our sample IDs match
+        # the observation IDs.
+        data_matrix = args[0]
+        if 0 in data_matrix.shape:
+            raise ValueError("The input data matrix must be at least 1x1 in "
+                             "size.")
+        if data_matrix.shape[0] != data_matrix.shape[1]:
+            raise ValueError("The input distance matrix must be square.")
+        if self.SampleIds != self.ObservationIds:
+            raise ValueError("The sample IDs must match the observation IDs.")
+        self._size = data_matrix.shape[0]
+
+    @property
+    def Size(self):
+        """Returns the size of the distance matrix (number of rows or columns).
+
+        This exists to explicitly make Size read-only by not having an
+        accompanying setter.
+        """
+        return self._size
+
+    @property
+    def DataMatrix(self):
+        """Returns the matrix of distances as a numpy array.
+        
+        The returned matrix is not a copy of the matrix stored in this object.
+        """
+        return asarray(self._data)
+
+    def max(self):
+        """Returns the maximum value present in the distance matrix.
+
+        Since distance matrices are guaranteed to be at least 1x1 in size, this
+        method will always return a valid maximum.
+        """
+        max_val = self[0][0]
+        for row_idx in range(self.Size):
+            for col_idx in range(self.Size):
+                if self[row_idx][col_idx] > max_val:
+                    max_val = self[row_idx][col_idx]
+        return max_val
+
+    def flatten(self, lower=True):
+        """Returns a list containing the flattened distance matrix.
+
+        The returned list will contain the elements in column-major order
+        (i.e. from leftmost to rightmost column, starting from the first row).
+
+        Arguments:
+            lower - If True, only the lower triangular elements will be
+                included (the diagonal will not be included). If False, all
+                elements (including the diagonal) will be included
+        """
+        flattened = []
+        for col_num in range(self.Size):
+            for row_num in range(self.Size):
+                if lower:
+                    if col_num < row_num:
+                        flattened.append(self[row_num][col_num])
+                else:
+                    flattened.append(self[row_num][col_num])
+        return flattened
+
+
+class MetadataMap():
+    """This class represents a QIIME metadata mapping file.
+    
+    Public attributes:
+        Comments - the comments associated with this metadata map (a list of
+            strings)
+    """
+
+    @staticmethod
+    def parseMetadataMap(lines):
+        """Parses a QIIME metadata mapping file into a MetadataMap object.
+
+        This static method is basically a factory that reads in the given
+        metadata mapping file contents and returns a MetadataMap instance. This
+        method is provided for convenience.
+
+        Arguments:
+            lines - a list of strings representing the file contents of a QIIME
+                metadata mapping file
+        """
+        return MetadataMap(*parse_mapping_file_to_dict(lines))
+
+    def __init__(self, sample_metadata, Comments):
+        """Instantiates a MetadataMap object.
+
+        Arguments:
+            sample_metadata - the output of parse_mapping_file_to_dict(). It
+                expects a python dict of dicts, where the top-level key is
+                sample ID, and the inner dict maps category name to category
+                value. This can be an empty dict altogether or the inner dict
+                can be empty
+            Comments - the output of parse_mapping_file_to_dict(). It expects a
+                list of strings for the comments in the mapping file. Can be an
+                empty list
+        """
+        self._metadata = sample_metadata
+        self.Comments = Comments
+
+    def __eq__(self, other):
+        """Test this instance for equality with another.
+
+        Note: This code was taken from http://stackoverflow.com/questions/
+            390250/elegant-ways-to-support-equivalence-equality-in-python-
+            classes.
+        """
+        if isinstance(other, self.__class__):
+            return self.__dict__ == other.__dict__
+        else:
+            return False
+
+    def __ne__(self, other):
+        """Test this instance for inequality with another.
+
+        Note: This code was taken from http://stackoverflow.com/questions/
+            390250/elegant-ways-to-support-equivalence-equality-in-python-
+            classes.
+        """
+        return not self.__eq__(other)
+
+    def getSampleMetadata(self, sample_id):
+        """Returns the metadata associated with a particular sample.
+
+        The metadata will be returned as a dict mapping category name to
+        category value.
+
+        Arguments:
+            sample_id - the sample ID (string) to retrieve metadata for
+        """
+        return self._metadata[sample_id]
+
+    def getCategoryValue(self, sample_id, category):
+        """Returns the category value associated with a sample's category.
+
+        The returned category value will be a string.
+
+        Arguments:
+            sample_id - the sample ID (string) to retrieve category information
+                for
+            category - the category name whose value will be returned
+        """
+        return self._metadata[sample_id][category]
+
+    def getCategoryValues(self, sample_ids, category):
+        """Returns all the values of a given category.
+
+        The return categories will be a list.
+
+        Arguments:
+            sample_ids - An ordered list of sample IDs (i.e., from a distance
+                matrix)
+            category - the category name whose values will be returned
+        """
+        return [self._metadata[sid][category] for sid in sample_ids]
+
+    @property
+    def SampleIds(self):
+        """Returns the IDs of all samples in the metadata map.
+
+        The sample IDs are returned as a list of strings in alphabetical order.
+        """
+        return sorted(self._metadata.keys())
+
+    @property
+    def CategoryNames(self):
+        """Returns the names of all categories in the metadata map.
+
+        The category names are returned as a list of strings in alphabetical
+        order.
+        """
+        return sorted(self.getSampleMetadata(self.SampleIds[0]).keys()) \
+            if len(self.SampleIds) > 0 else []
+
+
+class RExecutor(CommandLineApplication):
+    """RExecutor application controller
+       Runs R with a source script (from qiime/support_files/R)
+    """
+    _input_handler = '_input_as_path'
+    _command = "R"
+    _options ={}
+
+    _R_parameters = {
+        'flags': '--slave'
+        }
+
+    # The name of the R script (located under qiime/support_files/R/)
+    _R_script = ''
+
+    _parameters = {}
+    _parameters.update(_options)
+    _parameters.update(_R_parameters)
+
+    def getHelp(self):
+        """Returns documentation string"""
+        help_str =\
+        """
+        Runs the specified r script using the specified command
+        
+        Outputs:
+            The results of the r script that is ran
+        """
+        return help_str
+
+    def __call__(self, command_args, script_name, output_dir=None,
+                 verbose=False):
+        """Run the specified r script using the commands_args
+            
+            returns a CommandLineAppResult object
+        """
+        input_handler = self.InputHandler
+        suppress_stdout = self.SuppressStdout
+        suppress_stderr = self.SuppressStderr
+        if suppress_stdout:
+            outfile = devnull
+        else:
+            outfilepath = FilePath(join(self.TmpDir,'R.stdout'))
+            outfile = open(outfilepath,'w')
+        if suppress_stderr:
+            errfile = devnull
+        else:
+            errfilepath = FilePath(join(self.TmpDir,'R.stderr'))
+            errfile = open(errfilepath, 'w')
+
+        self._R_script = script_name
+        rscript = self._get_R_script_path()
+        base_command = self._get_base_command()
+        cd_command, base_command = base_command.split(';')
+        cd_command += ';'
+        R_source_dir = self._get_R_script_dir()
+
+        # Build up the command, consisting of a BaseCommand followed by
+        # input and output (file) specifications
+        command = self._commandline_join(
+            [   cd_command, base_command,
+                '--args',
+                '--source_dir', R_source_dir,
+            ] + command_args + [' < %s ' %(rscript)]
+            )
+
+        if self.HaltExec: 
+            raise AssertionError, "Halted exec with command:\n" + command
+
+        # run command, wait for output, get exit status
+        proc = Popen(command, shell=True, stdout=outfile, stderr=errfile)
+        proc.wait()
+        exit_status = proc.returncode
+
+        # Determine if error should be raised due to exit status of 
+        # appliciation
+        if not self._accept_exit_status(exit_status):
+            if exit_status == 2:
+                raise ApplicationError, \
+                    'R library not installed: \n' + \
+                    ''.join(open(errfilepath,'r').readlines()) + '\n'
+            else:
+                raise ApplicationError, \
+                    'Unacceptable application exit status: %s, command: %s'\
+                    % (str(exit_status),command) +\
+                    ' Program output: \n\n%s\n'\
+                     %(''.join(open(errfilepath,'r').readlines()))
+        # open the stdout and stderr if not being suppressed
+        out = None
+        if not suppress_stdout:
+            out = open(outfilepath,"r")
+        err = None
+        if not suppress_stderr:
+            err = open(errfilepath,"r")
+        if verbose:
+            msg = '\n\nCommand Executed: %s' % command + \
+                  ' \n\nR Command Output:\n%s' % \
+                  (''.join(open(errfilepath,'r').readlines()))
+            print(msg)
+
+    # The methods below were taken from supervised_learning.py
+    def _get_R_script_dir(self):
+        """Returns the path to the qiime R source directory."""
+        qiime_dir = get_qiime_project_dir()
+        script_dir = join(qiime_dir,'qiime','support_files','R')
+        return script_dir
+
+    def _get_R_script_path(self):
+        """Returns the path to the R script to be executed."""
+        return join(self._get_R_script_dir(), self._R_script)
+
+    def _commandline_join(self, tokens):
+        """Formats a list of tokens as a shell command."""
+        commands = filter(None, map(str, tokens))
+        return self._command_delimiter.join(commands).strip()
+
+    def _accept_exit_status(self,exit_status):
+        """ Return False to raise an error due to exit_status !=0."""
+        if exit_status != 0:
+            return False
+        return True
+
+    @property
+    def RParameters(self):
+        return self.__extract_parameters('R')
+
+    def __extract_parameters(self, name):
+        """Extracts parameters in self._<name>_parameters from self.Parameters.
+
+        Allows the program to conveniently access a subset of user-
+        adjusted parameters, which are stored in the Parameters
+        attribute.
+        
+        Relies on the convention of providing dicts named according to
+        "_<name>_parameters" and "_<name>_synonyms".  The main
+        parameters object is expected to be initialized with the
+        contents of these dicts.  This method will throw an exception
+        if either convention is not adhered to.
+        """
+        parameters = getattr(self, '_' + name + '_parameters')
+        result = Parameters(parameters)
+        for key in result.keys():
+            result[key] = self.Parameters[key]
+        return result
+
+
+def get_duplicates(fields):
+    """ Returns duplicates out of a list
+    
+    Modified from stackoverflow.com example duplicate detection code
+    http://stackoverflow.com/a/5420328
+    
+    fields:  list of elements to check for duplicates
+    """
+    cnt= Counter(fields)
+    return [key for key in cnt.keys() if cnt[key]> 1]
+
+def duplicates_indices(fields):
+    """ Gets dictionary of duplicates:locations in a list
+    
+    Modified from stackoverflow.com example duplicate detection code
+    http://stackoverflow.com/a/5420328
+    
+    fields:  list of elements to check for duplicates
+    """
+    dup, ind = get_duplicates(fields), defaultdict(list)
+    for i, v in enumerate(fields):
+        if v in dup: ind[v].append(i)
+    return ind
