@@ -12,12 +12,12 @@ from numpy import array
 from cogent.util.misc import safe_md5
 from cogent.parse.fasta import MinimalFastaParser
 from qiime.parse import parse_mapping_file, parse_qiime_parameters
-from qiime.format import format_otu_table
 from qiime.util import (compute_seqs_per_library_stats,
                         get_qiime_scripts_dir,
                         create_dir, guess_even_sampling_depth,
                         get_interesting_mapping_fields,qiime_system_call,
                         get_qiime_library_version)
+from biom.parse import parse_biom_table
 from cogent.core.moltype import IUPAC_DNA_ambiguities
 import os
 
@@ -25,7 +25,7 @@ __author__ = "Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Greg Caporaso", "Kyle Bittinger", "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -379,7 +379,7 @@ def run_pick_otus_through_otu_table(input_fp,
     commands.append([('Assign taxonomy',assign_taxonomy_cmd)])
     
     # Prep the OTU table building command
-    otu_table_fp = '%s/otu_table.txt' % output_dir
+    otu_table_fp = '%s/otu_table.biom' % output_dir
     try:
         params_str = get_params_str(params['make_otu_table'])
     except KeyError:
@@ -391,7 +391,7 @@ def run_pick_otus_through_otu_table(input_fp,
     commands.append([('Make OTU table', make_otu_table_cmd)])
     
     if cluster_failures:
-        reference_otu_table_fp = '%s/reference_only_otu_table.txt' % output_dir
+        reference_otu_table_fp = '%s/reference_only_otu_table.biom' % output_dir
         # Build the OTU table building command
         make_otu_table_cmd = '%s %s/make_otu_table.py -i %s -t %s -o %s %s' %\
          (python_exe_fp, script_dir, reference_otu_fp, taxonomy_fp, 
@@ -413,17 +413,20 @@ def run_pick_otus_through_otu_table(input_fp,
         except KeyError:
             params_str = ''
         
-        # Grab the OTU picker parameters
+        # Grab the alignment parameters        
+        # Want to find a cleaner strategy for this: the parallel script
+        # is method-specific, so doesn't take a --alignment_method
+        # option. This works for now though.
         try:
-            # Want to find a cleaner strategy for this: the parallel script
-            # is method-specific, so doesn't take a --alignment_method
-            # option. This works for now though.
             d = params['align_seqs'].copy()
+        except KeyError:
+            d = {}
+        try:
             del d['alignment_method']
-            params_str += ' %s' % get_params_str(d)
         except KeyError:
             pass
-            
+        params_str += ' %s' % get_params_str(d)
+        
         # Build the parallel pynast alignment command
         align_seqs_cmd = '%s %s/parallel_align_seqs_pynast.py -i %s -o %s -T %s' %\
          (python_exe_fp, script_dir, rep_set_fp, pynast_dir, params_str)
@@ -537,10 +540,11 @@ def run_pick_reference_otus_through_otu_table(
         # Grab the OTU picker parameters
         try:
             # Want to find a cleaner strategy for this: the parallel script
-            # is method-specific, so doesn't take a --otu_picking_method
+            # is method-specific, so doesn't take a --alignment_method
             # option. This works for now though.
             d = params['pick_otus'].copy()
-            del d['otu_picking_method']
+            if 'otu_picking_method' in d:
+                del d['otu_picking_method']
             params_str += ' %s' % get_params_str(d)
         except KeyError:
             pass
@@ -576,7 +580,7 @@ def run_pick_reference_otus_through_otu_table(
     commands.append([('Pick OTUs', pick_otus_cmd)])
 
     # Prep the OTU table building command
-    otu_table_fp = '%s/otu_table.txt' % pick_otu_dir
+    otu_table_fp = '%s/otu_table.biom' % pick_otu_dir
     try:
         params_str = get_params_str(params['make_otu_table'])
     except KeyError:
@@ -729,8 +733,8 @@ def run_beta_diversity_through_plots(otu_table_fp, mapping_fp,
              [('Beta Diversity (%s)' % beta_diversity_metric, beta_div_cmd)])
         
         
-        orig_beta_div_fp = '%s/%s_%s' % \
-         (output_dir, beta_diversity_metric, otu_table_filename)
+        orig_beta_div_fp = '%s/%s_%s.txt' % \
+         (output_dir, beta_diversity_metric, otu_table_basename)
         beta_div_fp = '%s/%s_dm.txt' % \
          (output_dir, beta_diversity_metric)
         commands.append([('Rename distance matrix (%s)' % beta_diversity_metric,
@@ -897,7 +901,7 @@ def run_qiime_alpha_rarefaction(otu_table_fp, mapping_fp,
     
     # Prep the rarefaction command
     try:
-        otu_table_f = open(otu_table_fp,'U')
+        open(otu_table_fp,'U')
     except IOError,e:
         logger.write('OTU table filepath cannot be opened. Does it exist?\n' +
                      ' %s\n' % otu_table_fp +
@@ -906,7 +910,7 @@ def run_qiime_alpha_rarefaction(otu_table_fp, mapping_fp,
         raise IOError,e
     if max_rare_depth == None:
         min_count, max_count, median_count, mean_count, counts_per_sample =\
-         compute_seqs_per_library_stats(otu_table_f)
+         compute_seqs_per_library_stats(parse_biom_table(open(otu_table_fp,'U')))
         max_rare_depth = median_count
     step = int((max_rare_depth - min_rare_depth) / num_steps) or 1
     max_rare_depth = int(max_rare_depth)
@@ -1414,7 +1418,7 @@ def run_core_qiime_analyses(
     
     if sampling_depth == None:
         min_count, max_count, median_count, mean_count, counts_per_sample =\
-         compute_seqs_per_library_stats(open(otu_table_fp))
+         compute_seqs_per_library_stats(parse_biom_table(open(otu_table_fp)))
         if even_sampling_keeps_all_samples:
             sampling_depth = min_count
         else:
@@ -1616,7 +1620,7 @@ def run_core_qiime_analyses(
     generate_index_page(index_links,index_fp)
 
 def run_summarize_taxa_through_plots(otu_table_fp, mapping_fp,
-    output_dir, mapping_cat, command_handler, params, qiime_config,
+    output_dir, mapping_cat, sort, command_handler, params, qiime_config,
     logger=None, status_update_callback=print_to_stdout):
     """ Run the data preparation for summarizing taxonomies and generating plots
     
@@ -1676,7 +1680,7 @@ def run_summarize_taxa_through_plots(otu_table_fp, mapping_fp,
         params_str = ''
     
     if mapping_cat:
-        output_fp=join(output_dir,'%s_otu_table.txt' % (mapping_cat))
+        output_fp=join(output_dir,'%s_otu_table.biom' % (mapping_cat))
         # Build the summarize otu by category command
         summarize_otu_by_cat_cmd = \
          "%s %s/summarize_otu_by_cat.py -i %s -c %s -o %s -m %s %s" %\
@@ -1687,7 +1691,36 @@ def run_summarize_taxa_through_plots(otu_table_fp, mapping_fp,
          [('Summarize OTU table by Category',summarize_otu_by_cat_cmd)])
          
         otu_table_fp=output_fp
-     
+    
+    # Build the sort OTU table command
+    if sort:
+        # Prep the sort_otu_table command
+        try:
+            params_str = get_params_str(params['sort_otu_table'])
+        except:
+            params_str = ''
+            
+        # define output otu table
+        sorted_fp=join(output_dir,
+                       splitext(split(otu_table_fp)[-1])[0]+'_sorted.biom')
+        
+        if mapping_cat or params_str=='':
+            # for this case we don't have a collapsed mapping file so must
+            # handle separately
+            sort_otu_table_cmd = \
+             "%s %s/sort_otu_table.py -i %s -o %s" %\
+             (python_exe_fp, script_dir, otu_table_fp, sorted_fp)
+        else:
+            sort_otu_table_cmd = \
+             "%s %s/sort_otu_table.py -i %s -o %s -m %s %s" %\
+             (python_exe_fp, script_dir, otu_table_fp, sorted_fp, \
+              mapping_fp, params_str)
+        
+        commands.append([('Sort OTU Table',sort_otu_table_cmd)])
+
+        # redefine otu_table_fp to use
+        otu_table_fp=sorted_fp
+    
     # Prep the summarize taxonomy command
     try:
         params_str = get_params_str(params['summarize_taxa'])
@@ -1894,7 +1927,7 @@ def run_ampliconnoise(mapping_fp,
             sample_name+"_pyronoise.pnout"
         commands.append([('pyronoise '+sample_name, cmd)])
 
-        cmd = 'Parse.pl '+bc_seqs[i]+' '+truncate_len+' < '+\
+        cmd = 'Parse.pl '+bc_seqs[i]+one_primer+' '+truncate_len+' < '+\
             sample_name+'_pyronoise_cd.fa'+' > '+ sample_name+'_'+\
             truncate_len+'.fa'
         commands.append([('truncate '+sample_name, cmd)])
