@@ -4,9 +4,9 @@ from __future__ import division
 
 __author__ = "Meg Pirrung"
 __copyright__ = "Copyright 2011, The QIIME project"
-__credits__ = ["Meg Pirrung"]
+__credits__ = ["Meg Pirrung", "Jesse Stombaugh"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Meg Pirrung"
 __email__ = "meganap@gmail.com"
 __status__ = "Release"
@@ -14,32 +14,30 @@ __status__ = "Release"
 from os.path import split,splitext
 from qiime.util import parse_command_line_parameters, make_option
 from qiime.util import load_qiime_config, create_dir
-from qiime.parse import parse_otu_table
+from biom.parse import parse_biom_table
 import os
 
 script_info = {}
 script_info['brief_description'] = "Makes TopiaryExplorer project file"
 script_info['script_description'] = "This script makes a TopiaryExplorer project file (.tep) and a jnlp file with the data location preloaded.\n\nWARNING: The jnlp file relies on an absolute path, if you move the .tep file, the generated jnlp will no longer work. However, you can still open the .tep file from your normal TopiaryExplorer install."
-script_info['script_usage'] = []
-script_info['script_usage'].append(("Example","Create .tep file and .jnlp file:","make_tep.py -i otu_table.txt -m mapping_file.txt -t otus.tre -o my_data"))
-# script_info['script_usage'].append(("""""","Create .tep file and .jnlp file, then launch TopiaryExplorer:","make_tep.py -i otu_table.txt -m mapping_file.txt -t otus.tre -o my_data -l"))
-script_info['output_description']= "The result of this script is written to a .tep file and a .jnlp file, both with the name supplied by -o"
-script_info['required_options'] = [\
- # Example required option
- make_option('-i','--otu_file',type="string",help='Path to read otu table',dest='otu_table_fp'),\
- make_option('-m','--mapping_file',type="string",help='Path to read mapping file',dest='mapping_fp'),\
- make_option('-t','--tree_file',type="string",help='Path to read tree',dest='tree_fp')
-]
-script_info['optional_options'] = [\
- # Example optional option
- make_option('-o','--output_dir',type="new_dirpath",help='the output directory [default: %default]', dest='out_fp'),\
- make_option('-p','--prefs_file_dir',type="string",help='Path to prefs file', dest='prefs_file_fp'),\
 
- # make_option('--create_jnlp',action='store_true',
- #  help='create a jnlp file [default: %default]'),\
-  make_option('-w','--web',action='store_true',default=False, help='web codebase jnlp flag [default: %default]', dest='web_flag'),
-  make_option('-u','--url_path',type="string",help='url path', dest='url')
- # make_option('-l','--launch_TE',action='store_false', default="false",help='Option to launch TopiaryExplorer [default: %default]', dest='launch')
+script_info['script_usage'] = []
+
+script_info['script_usage'].append(("Example","Create .tep file and .jnlp file:","%prog -i otu_table.biom -m Fasting_Map.txt -t rep_set.tre"))
+
+script_info['output_description']= "The result of this script is written to a .tep file and a .jnlp file, both with the name supplied by -o"
+
+script_info['required_options'] = [
+ make_option('-i','--otu_table_fp',type="existing_filepath",help='path to otu table in biom format'),\
+ make_option('-m','--mapping_fp',type="existing_filepath",help='path to mapping file'),\
+ make_option('-t','--tree_fp',type="existing_filepath",help='path to tree')
+]
+
+script_info['optional_options'] = [
+ make_option('-o','--out_fp',type="new_dirpath",help='the output directory [default: %default]'),\
+ make_option('-p','--prefs_file_fp',type="existing_filepath",help='path to prefs file'),\
+  make_option('-w','--web_flag',action='store_true',default=False, help='web codebase jnlp flag [default: %default]'),
+  make_option('-u','--url',type="string",help='url path')
 ]
 script_info['version'] = __version__
 
@@ -66,7 +64,7 @@ jnlp_middle_block = """\">
   <resources>
     <j2se version="1.6+" initial-heap-size="500M" max-heap-size="2000m" />
     
-    <jar href="topiaryexplorer0.9.4.jar" />
+    <jar href="topiaryexplorer0.9.6.jar" />
     <jar href="lib/core.jar" />
     <jar href="lib/itext.jar" />
     <jar href="lib/pdf.jar" />
@@ -162,24 +160,29 @@ def main():
     tep_fp = '%s/%s.tep' % (output_dir,output_basename)      # opts.out_fp+'.tep'
     jnlp_fp = '%s/%s.jnlp' % (output_dir,output_basename)
     tepfile = open(tep_fp, 'w')
-    otu_lines = open(otu_table_fp, 'U').readlines()
-    sample_ids, otu_ids, otu_table, metadata = parse_otu_table(otu_lines)
+    otu_table_data = parse_biom_table(open(otu_table_fp, 'U'))
+    #sample_ids, otu_ids, otu_table, metadata = parse_otu_table(otu_lines)
     mapping_lines = open(mapping_fp, 'U')    
     tree_lines = open(tree_fp, 'U')
     
     lines = ['>>tre\n']
     lines += tree_lines.readlines() 
     lines += '\n'
-    if(metadata):
+    if(otu_table_data.ObservationMetadata):
         lines += '>>otm\n#OTU ID\tOTU Metadata\n'
-        for i in range(len(otu_ids)):
-            lines += otu_ids[i] + '\t'
-            for m in metadata[i]:
+        for i in range(len(otu_table_data.ObservationIds)):
+            lines += otu_table_data.ObservationIds[i] + '\t'
+            for m in otu_table_data.ObservationMetadata[i]['taxonomy']:
                 lines += m + ';'
             # lines = lines[:len(lines)-1]
             lines += '\n'
     lines += '>>osm\n'
-    lines += otu_lines
+    if otu_table_data.ObservationMetadata is None:
+        lines += otu_table_data.delimitedSelf()
+    elif "taxonomy" in otu_table_data.ObservationMetadata[0]:
+        lines += otu_table_data.delimitedSelf(header_key="taxonomy", 
+                                       header_value="Consensus Lineage",
+                                       metadata_formatter=lambda x: ';'.join(x))
     lines += '\n>>sam\n'
     lines += mapping_lines.readlines()
     
@@ -200,20 +203,18 @@ def main():
     else:
         topiaryexplorer_project_dir =\
          load_qiime_config()['topiaryexplorer_project_dir']
-        if topiaryexplorer_project_dir == None:
-            option_parser.error("Couldn't create jnlp file - topiaryexplorer_project_dir is not defined in your qiime_config. tep file was created sucessfully.")
-            exit(0)
+        if topiaryexplorer_project_dir:
+            lines += 'file:' + topiaryexplorer_project_dir    
+            lines += jnlp_middle_block
+            if(opts.url):
+                lines += opts.url
+            else:
+                lines += os.path.abspath(tep_fp)
+            # lines += os.path.abspath(tep_fp)
+            lines += jnlp_bottom_block
+            jnlpfile.writelines(lines)
         else:
-            lines += 'file:' + topiaryexplorer_project_dir
-    
-    lines += jnlp_middle_block
-    if(opts.url):
-        lines += opts.url
-    else:
-        lines += os.path.abspath(tep_fp)
-    # lines += os.path.abspath(tep_fp)
-    lines += jnlp_bottom_block
-    jnlpfile.writelines(lines)
+            print "WARNING: Couldn't create jnlp file - topiaryexplorer_project_dir is not defined in your qiime_config. tep file was created sucessfully."
     
     
     
