@@ -4,27 +4,29 @@ from __future__ import division
 
 __author__ = "Doug Wendel"
 __copyright__ = "Copyright 2011, The QIIME Project"
-__credits__ = ["Catherine Lozupone", "Jesse Stombaugh", "Doug Wendel", "Dan Knights"]
+__credits__ = ["Catherine Lozupone", "Jesse Stombaugh", "Doug Wendel", "Dan Knights", "Greg Caporaso"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Doug Wendel"
 __email__ = "wendel@colorado.edu"
 __status__ = "Release"
  
 
+from os.path import isdir, join
+from os import listdir
+from glob import glob
 from qiime.otu_category_significance import test_wrapper, test_wrapper_multiple
 from qiime.longitudinal_otu_category_significance import \
     longitudinal_otu_table_conversion_wrapper
 from qiime.util import parse_command_line_parameters
 from qiime.util import make_option
-from os.path import isdir, join
-from os import listdir
+from qiime.format import format_biom_table
 
 script_info={}
 script_info['brief_description']="""OTU significance and co-occurence analysis"""
-script_info['script_description']="""The script otu_category_significance.py tests whether any of the OTUs in an OTU table are significantly associated with a category in the category mapping file. This code uses, ANOVA, the G test of independence, Pearson correlation, or a paired T test to find OTUs whose members are differentially represented across experimental treatments or measured variables.
+script_info['script_description']="""The script otu_category_significance.py tests whether any of the OTUs in an OTU table are significantly associated with a category in the category mapping file. This code uses ANOVA, the G test of independence, Pearson correlation, or a paired t-test to find OTUs that are differentially represented across experimental treatments or measured variables.
 
-The script can also be used to measure co-occurrence. For instance it can also be used with presence/absence or abundance data for a phylogenetic group (such as that determined with quantitative PCR) to determine if any OTUs co-occur with a taxon of interest, using the ANOVA or G test of Independence.
+The script can also be used to measure co-occurrence. For instance it can also be used with presence/absence or abundance data for a phylogenetic group (such as that determined with quantitative PCR) to determine if any OTUs co-occur with a taxon of interest, using the ANOVA, G test of Independence, or correlation.
 
 The statistical test to be run is designated with the -s option, and includes the following options:
 
@@ -39,15 +41,21 @@ The tests also include options for longitudinal data (i.e. datasets in which mul
 
 Pearson correlation (longitudinal_correlation): determines whether OTU relative abundance is correlated with a continuous variable in the category mapping file while accounting for an experimental design where multiple samples are collected from the same individual or site. Uses the change in relative abundance for each sample from the reference sample (e.g. timepoint zero in a timeseries analysis) rather than the absolute relative abundances in the correlation (e.g. if the relative abundance before the treatment was 0.2, and after the treatment was 0.4, the new values for the OTU relative abundance will be 0.0 for the before sample, and 0.2 for the after, thus indicating that the OTU went up in response to the treatment.)
 
-Paired T-test (paired_T): This option is when measurements were taken "before" and "after" a treatment. There must be exactly two measurements for each individual/site. The category mapping file must again have an individual column, indicating which sample is from which individual, and a reference_sample column, that has a 1 for the before time point and a 0 for the after.
+Paired t-test (paired_T): This option is when measurements were taken "before" and "after" a treatment. There must be exactly two measurements for each individual/site. The category mapping file must again have an individual column, indicating which sample is from which individual, and a reference_sample column, that has a 1 for the before time point and a 0 for the after.
 
 With the exception of longitudinal correlation and paired_T, this script can be performed on a directory of OTU tables (for example, the output of multiple_rarefactions_even_depth.py), in addition to on a single OTU table. If the script is called on a directory, the resulting p-values are the average of the p-values observed when running a single test on each otu_table separately. It is generally a good practice to rarefy the OTU table (e.g. with single_rarefaction.py) prior to running these significance tests in order to avoid artifacts/biases from unequal sample sizes.
 """
 script_info['script_usage']=[]
-script_info['script_usage'].append(("Example 1","""If the user would like to perform a G test on their OTU table using default parameters, while testing the category "Sex", they can run the following command:""","""otu_category_significance.py -i otu_table.txt -m Mapping_file.txt -s g_test -c Sex"""))
-script_info['script_usage'].append(("Example 2","""If the user would like to perform the same test using numerical qPCR data, where everything below a threshold value should be considered "absent" and everything above that value "present", the user will need to set the threshold by running the following command:""","""otu_category_significance.py -i otu_table.txt -m Mapping_file.txt -s g_test -c qPCR -t 0.16"""))
-script_info['script_usage'].append(("Example 3","""Alternatively, the user could run an ANOVA test on the same data by using the following command:""","""otu_category_significance.py -i otu_table.txt -m Mapping_file.txt -s ANOVA -c Sex"""))
-script_info['script_usage'].append(("Example 4","""If the user would like to perform an ANOVA on an entire directory of rarefied  OTU tables using default parameters, while testing the category "Sex", they can run the following command:""","""otu_category_significance.py -i otu_table_dir -m Mapping_file.txt -s g_test -c Sex"""))
+
+script_info['script_usage'].append(("G-test","""Perform a G test on otu_table.biom testing OTUs that occur in 4 or more samples (-f) for differences in the abundance across the category "Treatment":""","""%prog -i otu_table.biom -m Fasting_Map.txt -s g_test -c Treatment -f 4 -o single_g_test.txt"""))
+
+script_info['script_usage'].append(("ANOVA","""Perform an ANOVA on otu_table.biom testing OTUs that occur in 4 or more samples (-f) for differences in the abundance across the category "Treatment":""","""%prog -i otu_table.biom -m Fasting_Map.txt -s ANOVA -c Treatment -f 4 -o single_anova.txt"""))
+
+script_info['script_usage'].append(("ANOVA on mutliple OTU tables","""Perform an ANOVA on all OTU tables in rarefied_otu_tables testing OTUs that occur in 4 or more samples (-f) for differences in the abundance across the category "Treatment":""","""%prog -i rarefied_otu_tables -m Fasting_Map.txt -s ANOVA -c Treatment -f 4 -o multiple_anova.txt"""))
+
+# do we have good input data for this? 
+# script_info['script_usage'].append(("Example 2","""If the user would like to perform the same test using numerical qPCR data, where everything below a threshold value should be considered "absent" and everything above that value "present", the user will need to set the threshold by running the following command:""","""%prog -i otu_table.biom -m Mapping_file.txt -s g_test -c Treatment -t 0.16"""))
+
 script_info['output_description']="""The G test results are output as tab delimited text, which can be examined in Excel. The output has the following columns:
 
 * OTU: The name of the OTU.
@@ -92,10 +100,10 @@ The paired_T results are output as tab delimited text that can be examined in Ex
 """
 
 script_info['required_options']=[\
-    make_option('-i','--otu_table_fp', dest='otu_table_fp',\
-        help='path to the otu table, or to a directory containing OTU tables'),\
-    make_option('-m','--category_mapping_fp',\
-        dest='category_mapping_fp',\
+    make_option('-i','--otu_table_fp',\
+        help='path to the otu table in biom format, or to a directory ' + \
+             'containing OTU tables',type='existing_path'),\
+    make_option('-m','--category_mapping_fp',type='existing_filepath',
         help='path to category mapping file')
 ]
 
@@ -114,13 +122,13 @@ script_info['optional_options']=[\
         'abundance is correlated with a continuous variable in the ' +\
         'category mapping file in longitudinal study designs such as ' +\
         'with timeseries data.     paired_T: determine whether OTU ' +\
-        'relative abundance goes up or down in response to a treatment.',
+        'relative abundance goes up or down in response to a treatment. [default: %default]',
         type="choice",choices=["g_test", "ANOVA", "correlation", \
         "longitudinal_correlation", "paired_T"]),
     make_option('-o','--output_fp', dest='output_fp', \
         default= 'otu_category_significance_results.txt',\
-        help='path to output file. otu_category_significance_results.txt ' +\
-        'by default'),\
+        help='path to output file. [default: %default]',
+        type='new_filepath'),\
     make_option('-f','--filter', dest='filter',\
         default= 10, \
         help='minimum number of samples that must contain the OTU for the ' +\
@@ -137,7 +145,7 @@ script_info['optional_options']=[\
         'Only used if you have numerical data that should be converted to ' +\
         'present or absent based on a threshold. Should be None for ' +\
         'categorical data or with the correlation test. default value is None'),\
-    make_option('-l', '--otu_include_fp', dest='otu_include_fp', default=None,\
+    make_option('-l', '--otu_include_fp', type='existing_filepath', default=None,\
         help='path to a file with a list of OTUs to evaluate. By default ' +\
         'evaluates all OTUs that pass the minimum sample filter. If a ' +\
         'filepath is given here in which each OTU name one wishes to ' +\
@@ -153,14 +161,21 @@ script_info['optional_options']=[\
         default='individual',\
         help='name of the column in the category mapping file that ' +\
         'designates which sample is from which individual.'),\
-    make_option('-b', '--converted_otu_table_output_fp', dest='conv_output_fp',\
+    make_option('-b', '--converted_otu_table_output_fp', type='new_filepath',\
         default=None, help='the test options longitudinal_correlation and ' +\
         'paired_T convert the original OTU table into one in which samples ' +\
         'that are ignored because they are never observed in an individual ' +\
         'are replaced with the ignore number 999999999 and the OTU counts are ' +\
         'the change in relative abundance compared to the designated reference ' +\
         'sample. If a filepath is given with the -b option ' +\
-        'this converted OTU table will be saved to this path.')]
+        'this converted OTU table will be saved to this path.'),\
+    make_option('--relative_abundance', default=False, help=('Some of the '
+        'statistical tests, such as Pearson correlation and ANOVA, convert '
+        'the OTU counts to relative abundances prior to performing the '
+        'calculations. This parameter can be set if a user wishes to disable '
+        'this step. (e.g. if an OTU table has already been converted '
+        'to relative abundances.)'),\
+        action='store_true')]
 
 script_info['version'] = __version__
 
@@ -175,7 +190,8 @@ def main():
     category_mapping = open(category_mapping_fp,'U')
     individual_column = opts.individual_column
     reference_sample_column = opts.reference_sample_column
-    conv_output_fp = opts.conv_output_fp
+    conv_output_fp = opts.converted_otu_table_output_fp
+    relative_abundance = opts.relative_abundance
 
     filter = opts.filter
     test = opts.test
@@ -199,34 +215,35 @@ def main():
         # if single file, process normally
         otu_table = open(otu_table_fp,'U')
         if test == 'longitudinal_correlation' or test == 'paired_T':
-            converted_otu_table = longitudinal_otu_table_conversion_wrapper(\
-                otu_table, category_mapping, individual_column, \
-                reference_sample_column) 
+            converted_otu_table_str = format_biom_table(\
+                    longitudinal_otu_table_conversion_wrapper(otu_table,
+                            category_mapping, individual_column,
+                            reference_sample_column))
             if conv_output_fp:
                 of = open(conv_output_fp, 'w')
-                of.write(converted_otu_table)
+                of.write(converted_otu_table_str)
                 of.close()
-            converted_otu_table = converted_otu_table.split('\n')
             category_mapping = open(category_mapping_fp,'U')
             if test == 'longitudinal_correlation':
-                output = test_wrapper('correlation', converted_otu_table, \
+                output = test_wrapper('correlation', converted_otu_table_str, \
                     category_mapping, category, threshold, filter, otu_include, \
                     999999999.0, True)
             elif test == 'paired_T':
-                output = test_wrapper('paired_T', converted_otu_table, \
+                output = test_wrapper('paired_T', converted_otu_table_str, \
                     category_mapping, category, threshold, \
                     filter, otu_include, 999999999.0, True, \
                     individual_column, reference_sample_column)
         else:
             output = test_wrapper(test, otu_table, category_mapping, \
-                category, threshold, filter, otu_include)
+                category, threshold, filter, otu_include, \
+                otu_table_relative_abundance=relative_abundance)
     else:
         if test != 'longitudinal_correlation' and test != 'paired_T':
-            otu_table_paths = [join(otu_table_fp,fp) for fp in \
-                listdir(otu_table_fp)]
+            otu_table_paths = glob('%s/*biom' % otu_table_fp)
             # if directory, get aggregated results
             output = test_wrapper_multiple(test, otu_table_paths, \
-                category_mapping, category, threshold, filter, otu_include)
+                category_mapping, category, threshold, filter, otu_include,\
+                otu_table_relative_abundance=relative_abundance)
         else:
             raise ValueError("the longitudinal_correlation and paired_T options cannot be run on a directory")
         
