@@ -6,7 +6,7 @@ __author__ = "Jens Reeder"
 __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Greg Caporaso", "Jens Reeder"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -51,11 +51,11 @@ fragment3:  Archaea;Euryarchaeota;Methanobacteriales;Methanobacterium
 
 The sequence would be considered chimeric at a depth of 3 (Methanobacteriales vs. Halobacteriales), but non-chimeric at a depth of 2 (all Euryarchaeota).
 
-blast_fragments begins with the assumption that a sequence is non-chimeric, and looks for evidence to the contrary. This is important when, for example, no taxonomy assignment can be made because no blast result is returned. If a sequence is split into three fragments, and only one returns a blast hit, that sequence would be considered non-chimeric. This is because there is no evidence (i.e., contradictory blast assignments) for the sequence being chimeric. This script can be run by the following command, where the resulting data is written to the directory "identify_chimeras/" and using default parameters (e.g. chimera detection method ("-m blast_fragments"), number of fragments ("-n 3"), taxonomy depth ("-d 4") and maximum E-value ("-e 1e-30")):""","""%prog -i repr_set_seqs.fasta -t taxonomy_assignment.txt -r ref_seq_set.fna -o chimeric_seqs.txt"""))
+blast_fragments begins with the assumption that a sequence is non-chimeric, and looks for evidence to the contrary. This is important when, for example, no taxonomy assignment can be made because no blast result is returned. If a sequence is split into three fragments, and only one returns a blast hit, that sequence would be considered non-chimeric. This is because there is no evidence (i.e., contradictory blast assignments) for the sequence being chimeric. This script can be run by the following command, where the resulting data is written to $PWD/blast_fragments_chimeric_seqs.txt and using default parameters (i.e., number of fragments ("-n 3"), taxonomy depth ("-d 4") and maximum E-value ("-e 1e-30")). ALWAYS SPECIFY ABSOLUTE FILE PATHS (absolute path represented here as $PWD, but will generally look something like /home/ubuntu/my_analysis/).""","""%prog -i $PWD/inseqs.fasta -t $PWD/id_to_tax.txt -r $PWD/refseqs.fasta -o $PWD/blast_fragments_chimeric_seqs.txt -m blast_fragments"""))
 
 script_info['script_usage'].append(("""ChimeraSlayer Example:""",
-                                    """Identify chimeric sequences using the ChimeraSlayer algorithm against a user provided reference database. The input sequences need to be provided in aligned (Py)Nast format and the reference database needs to be provided as aligned FASTA (-a). Note that the reference database needs to be the same that was used to build the alignment of the input sequences!""",
-                                    """%prog -m ChimeraSlayer -i repr_set_seqs_aligned.fasta -a ref_seq_set_aligned.fasta -o chimeric_seqs.txt"""))
+                                    """Identify chimeric sequences using the ChimeraSlayer algorithm against a user provided reference database. The input sequences need to be provided in aligned (Py)Nast format and the reference database needs to be provided as aligned FASTA (-a). Note that the reference database needs to be the same that was used to build the alignment of the input sequences! ALWAYS SPECIFY ABSOLUTE FILE PATHS (absolute path represented here as $PWD, but will generally look something like /home/ubuntu/my_analysis/).""",
+                                    """%prog -i $PWD/inseqs_aligned.fasta -o $PWD/chimera_slayer_chimeric_seqs.txt"""))
                            
 script_info['output_description']="""The result of parallel_identify_chimeric_seqs.py is a text file that identifies which sequences are chimeric."""
 
@@ -67,15 +67,19 @@ chimera_detection_method_choices = ['blast_fragments','ChimeraSlayer']
 
 script_info['optional_options']=[\
     make_option('-a', '--aligned_reference_seqs_fp',
+        type='existing_filepath',
+        default=qiime_config['pynast_template_alignment_fp'],
         help='Path to (Py)Nast aligned reference sequences. '
         'REQUIRED when method ChimeraSlayer [default: %default]'), 
 
     make_option('-t', '--id_to_taxonomy_fp',
+        type='existing_filepath',
         help='Path to tab-delimited file mapping sequences to assigned '
          'taxonomy. Each assigned taxonomy is provided as a comma-separated '
          'list. [default: %default; REQUIRED when method is blast_fragments]'),
 
     make_option('-r', '--reference_seqs_fp',
+        type='existing_filepath',
         help='Path to reference sequences (used to build a blast db when method blast_fragments). '
         '[default: %default; REQUIRED when method blast_fragments'+\
          ' if no blast_db is provided;]'),
@@ -111,11 +115,12 @@ script_info['optional_options']=[\
                     ' [default: %default]', default=None),       
 
     make_option('-o', '--output_fp',
+        type='new_filepath',
         help='Path to store output [default: derived from input_seqs_fp]'),
           
     #Define parallel-script-specific parameters
     make_option('-N','--identify_chimeric_seqs_fp',action='store',\
-           type='string',help='full path to '+\
+           type='existing_filepath',help='full path to '+\
            'scripts/identify_chimeric_seqs.py [default: %default]',\
            default=join(get_qiime_scripts_dir(),'identify_chimeric_seqs.py')),\
         
