@@ -6,7 +6,7 @@ __author__ = "Antonio Gonzalez Pena"
 __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Antonio Gonzalez Pena"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Antonio Gonzalez Pena"
 __email__ = "antgonza@gmail.com"
 __status__ = "Release"
@@ -36,31 +36,10 @@ script_info['required_options']=[\
      help='path to distance matrix to be displayed in the x axis'),\
  make_option('-y', '--input_path_y',\
      help='path to distance matrix to be displayed in the y axis'),\
- make_option('-m', '--model', type='choice',\
-     choices=FitModel.options, default='exponential', 
-     help='model to be fitted to the data. Valid ' +\
-     'choices are:' + ', '.join(FitModel.options) + '. [default: %default]'),\
  make_option('-o', '--output_path',
      help='output path. directory for batch processing, '+\
        'filename for single file operation'),
- make_option('-X', '--x_label', default='Distance Dissimilarity (m)',\
-     help='Label for the x axis [default: %default]'),
- make_option('-Y', '--y_label', default='Community Dissimilarity',\
-     help='Label for the y axis [default: %default]'),
- make_option('-t', '--fig_title', default='Semivariogram',\
-     help='Title of the plot [default: %default]'),       
- make_option('--dot_color', help='dot color for plot, more info:' +\
-    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
-    ' [default: %default]', default="wo"), 
- make_option('--line_color', help='line color for plot, more info:' +\
-    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
-    ' [default: %default]', default="blue"), 
- make_option('--dot_alpha', type='float', help='alpha for dots, more info:' +\
-    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
-    ' [default: %default]', default=1),
- make_option('--line_alpha', type='float', help='alpha for dots, more info:' +\
-    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
-    ' [default: %default]', default=1),
+
 ]
 script_info['optional_options']=[\
  make_option('-b', '--binning', type='string',\
@@ -76,6 +55,34 @@ script_info['optional_options']=[\
  make_option('--x_min', type='float', help='x axis min limit [default: auto]', default=None),         
  make_option('--y_max', type='float', help='y axis max limit [default: auto]', default=None),         
  make_option('--y_min', type='float', help='y axis min limit [default: auto]', default=None),
+ make_option('-X', '--x_label', default='Distance Dissimilarity (m)',\
+     help='Label for the x axis [default: %default]'),
+ make_option('-Y', '--y_label', default='Community Dissimilarity',\
+     help='Label for the y axis [default: %default]'),
+ make_option('-t', '--fig_title', default='Semivariogram',\
+     help='Title of the plot [default: %default]'),       
+ make_option('--dot_color', help='dot color for plot, more info:' +\
+    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
+    ' [default: %default]', default="white"), 
+ make_option('--dot_marker', help='dot color for plot, more info:' +\
+    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
+    ' [default: %default]', default="o"), 
+ make_option('--line_color', help='line color for plot, more info:' +\
+    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
+    ' [default: %default]', default="blue"), 
+ make_option('--dot_alpha', type='float', help='alpha for dots, more info:' +\
+    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
+    ' [default: %default]', default=1),
+ make_option('--line_alpha', type='float', help='alpha for dots, more info:' +\
+    ' http://matplotlib.sourceforge.net/api/pyplot_api.html' +\
+    ' [default: %default]', default=1),
+ make_option('-m', '--model', type='choice',\
+     choices=FitModel.options, default='exponential', 
+     help='model to be fitted to the data. Valid ' +\
+     'choices are:' + ', '.join(FitModel.options) + '. [default: %default]'),\
+ make_option('-p', '--print_model', action='store_true',
+     help='Print in the title of the plot the function of the fit. ' +\
+     '[default: %default]',default=False)
 ]
 
 script_info['version'] = __version__
@@ -143,10 +150,10 @@ def main():
             raise ValueError, 'The distance matrices have different sizes. ' +\
                 'You can cancel this error by passing --ignore_missing_samples'
         
-    (x_val,y_val,x_fit,y_fit) =\
+    (x_val,y_val,x_fit,y_fit,func_text) =\
           fit_semivariogram((x_samples,x_distmtx), (y_samples,y_distmtx), opts.model, ranges)
     
-    plot(x_val, y_val, opts.dot_color, alpha=opts.dot_alpha)
+    plot(x_val, y_val, color=opts.dot_color, marker=opts.dot_marker, linestyle="None", alpha=opts.dot_alpha)
     plot(x_fit, y_fit, linewidth=2.0, color=opts.line_color, alpha=opts.line_alpha)
     
     if opts.x_min!=None and opts.x_max!=None:
@@ -160,7 +167,10 @@ def main():
     
     xlabel(x_label)
     ylabel(y_label)
-    title(fig_title)
+    if opts.print_model:
+        title(fig_title + ' ' + func_text)
+    else:
+        title(fig_title)
     
     savefig(opts.output_path)
 
