@@ -7,7 +7,7 @@ __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Jens Reeder","Dan Knights", "Antonio Gonzalez Pena",
     "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -33,15 +33,24 @@ from qiime.denoiser.utils import check_flowgram_ali_exe
 
 from cogent import __version__ as pycogent_lib_version
 from numpy import __version__ as numpy_lib_version
+
+try:
+    from biom import __version__ as biom_lib_version
+except ImportError:
+    biom_lib_version = "ERROR: Not installed - this is required!"
+
 try:
     from matplotlib import __version__ as matplotlib_lib_version
 except ImportError:
     matplotlib_lib_version = "Not installed."
+    
 try:
     from pynast import __version__ as pynast_lib_version
 except ImportError:
     pynast_lib_version = "Not installed."
 
+
+
 script_info = {}
 script_info['brief_description']= """Print out the qiime config settings."""
 script_info['script_description'] = """A simple scripts that prints out the qiime config settings and does some sanity checks."""
@@ -87,7 +96,25 @@ class Qiime_config(TestCase):
     def test_cluster_jobs_fp(self):
         """cluster_jobs_fp is set to a valid path and is executable"""       
         
-        test_qiime_config_variable("cluster_jobs_fp", self.config, self, X_OK,)
+        fp = self.config["cluster_jobs_fp"]
+        
+        if not fp:
+            self.fail("Your qiime_config file doesn't have cluster_jobs_fp\n.")
+        
+        full_path = app_path(fp)
+        if full_path:
+            fp = full_path
+        
+        #test if file exists or is in $PATH
+        self.assertTrue(exists(fp),
+         "cluster_jobs_fp set to an invalid file path or is not in $PATH: %s" % fp)
+
+        modes = {R_OK:"readable",
+                 W_OK:"writable",
+                 X_OK:"executable"}
+        #test if file readable    
+        self.assertTrue(access(fp, X_OK),
+            "cluster_jobs_fp is not %s: %s" % (modes[X_OK], fp))
    
     def test_blastmat_dir(self):
         """blastmat_dir is set to a valid path."""
@@ -143,6 +170,12 @@ class Qiime_config(TestCase):
             
         test_qiime_config_variable("pynast_template_alignment_fp",
                                    self.config, self)
+                                   
+    def test_qiime_test_data_dir(self):
+        """qiime_test_data_dir, if set, is set to a valid path"""
+            
+        test_qiime_config_variable("qiime_test_data_dir",
+                                   self.config, self)
             
     def test_pynast_template_alignment_blastdb_fp(self):
         """pynast_template_alignment_blastdb, if set, is set to a valid path"""
@@ -289,6 +322,8 @@ class Qiime_config(TestCase):
         stdout = proc.stdout.read()
         version_string = stdout.strip().split('Python')[-1].strip()
         try:
+            if version_string[-1]=='+':
+                version_string = version_string[:-1]
             version = tuple(map(int,version_string.split('.')))
             if len(version) == 2:
                 version = (version[0],version[1],0)
@@ -355,7 +390,7 @@ class Qiime_config(TestCase):
             pass_test = False
             version_string = "Not installed"
         self.assertTrue(pass_test,\
-         "Unsupported matplotlib version. Must be >= %s and < %s , but running %s." \
+         "Unsupported pynast version. Must be >= %s and < %s , but running %s." \
          % ('.'.join(map(str,min_acceptable_version)),
             '.'.join(map(str,min_unacceptable_version)),
             version_string))
@@ -467,13 +502,12 @@ class Qiime_config(TestCase):
         
     def test_mothur_supported_version(self):
         """mothur is in path and version is supported """
-        acceptable_version = (1,6,0)
+        acceptable_version = (1,25,0)
         self.assertTrue(app_path('mothur'),
          "mothur not found. This may or may not be a problem depending on "+\
          "which components of QIIME you plan to use.")
         # mothur creates a log file in cwd, so create a tmp and cd there first
-        tmp_dir = get_random_directory_name(output_dir='/tmp/')
-        command = "cd %s ; mothur -v | grep ^mothur" % tmp_dir
+        command = "mothur 'read.dist(help)' | grep '^mothur'"
         proc = Popen(command,shell=True,universal_newlines=True,\
                          stdout=PIPE,stderr=STDOUT)
         stdout, stderr = proc.communicate()
@@ -487,9 +521,6 @@ class Qiime_config(TestCase):
         self.assertTrue(pass_test,\
          "Unsupported mothur version. %s is required, but running %s." \
          % ('.'.join(map(str,acceptable_version)), version_string))
-        
-        # remove the directory and the log file
-        rmtree(tmp_dir)
          
     def test_denoiser_supported_version(self):
         """denoiser aligner is ready to use """
@@ -506,7 +537,7 @@ class Qiime_config(TestCase):
         
     def test_raxmlHPC_supported_version(self):
         """raxmlHPC is in path and version is supported """
-        acceptable_version = (7,0,3)
+        acceptable_version = [(7,3,0),(7,3,0)]
         self.assertTrue(app_path('raxmlHPC'),
          "raxmlHPC not found. This may or may not be a problem depending on "+\
          "which components of QIIME you plan to use.")
@@ -517,7 +548,7 @@ class Qiime_config(TestCase):
         version_string = stdout.strip().split(' ')[4].strip()
         try:
             version = tuple(map(int,version_string.split('.')))
-            pass_test = version == acceptable_version
+            pass_test = version in acceptable_version
         except ValueError:
             pass_test = False
             version_string = stdout
@@ -567,6 +598,110 @@ class Qiime_config(TestCase):
     #    #ChimeraSlayer does not have a version print in their program
     #    pass
         
+    def test_rtax_supported_version(self):
+        """rtax is in path and version is supported """
+        acceptable_version = [(0,981),(0,981)]
+        self.assertTrue(app_path('rtax'),
+         "rtax not found. This may or may not be a problem depending on "+\
+         "which components of QIIME you plan to use.")
+        command = "rtax 2>&1 > /dev/null | grep Version | awk '{print $2}'"
+        proc = Popen(command,shell=True,universal_newlines=True,\
+                         stdout=PIPE,stderr=STDOUT)
+        stdout = proc.stdout.read()
+        version_string = stdout.strip()
+        try:
+            version = tuple(map(int,version_string.split('.')))
+            pass_test = version in acceptable_version
+        except ValueError:
+            pass_test = False
+            version_string = stdout
+        self.assertTrue(pass_test,\
+         "Unsupported rtax version. %s is required, but running %s." \
+         % ('.'.join(map(str,acceptable_version)), version_string))
+         
+    def test_pplacer_supported_version(self):
+        """pplacer is in path and version is supported """
+        acceptable_version = [(1,1),(1,1)]
+        self.assertTrue(app_path('pplacer'),
+         "pplacer not found. This may or may not be a problem depending on "+\
+         "which components of QIIME you plan to use.")
+        command = "pplacer --version"
+        proc = Popen(command,shell=True,universal_newlines=True,\
+                         stdout=PIPE,stderr=STDOUT)
+        stdout = proc.stdout.read()
+        version_string = stdout.strip()[1:4]
+        try:
+            version = tuple(map(int,version_string.split('.')))
+            pass_test = version in acceptable_version
+        except ValueError:
+            pass_test = False
+            version_string = stdout
+        self.assertTrue(pass_test,\
+         "Unsupported pplacer version. %s is required, but running %s." \
+         % ('.'.join(map(str,acceptable_version)), version_string))
+         
+    def test_ParsInsert_supported_version(self):
+        """ParsInsert is in path and version is supported """
+        acceptable_version = ["1.04"]
+        self.assertTrue(app_path('ParsInsert'),
+         "ParsInsert not found. This may or may not be a problem depending on "+\
+         "which components of QIIME you plan to use.")
+        command = "ParsInsert -v | grep App | awk '{print $3}'"
+        proc = Popen(command,shell=True,universal_newlines=True,\
+                         stdout=PIPE,stderr=STDOUT)
+        stdout = proc.stdout.read()
+        version_string = stdout.strip()
+        try:
+            pass_test = version_string in acceptable_version
+        except ValueError:
+            pass_test = False
+            version_string = stdout
+        self.assertTrue(pass_test,\
+         "Unsupported ParsInsert version. %s is required, but running %s." \
+         % ('.'.join(map(str,acceptable_version)), version_string))
+        
+    def test_usearch_supported_version(self):
+        """usearch is in path and version is supported """
+        acceptable_version = [(5,2,32),(5,2,32)]
+        self.assertTrue(app_path('usearch'),
+         "usearch not found. This may or may not be a problem depending on "+\
+         "which components of QIIME you plan to use.")
+        command = "usearch --version"
+        proc = Popen(command,shell=True,universal_newlines=True,\
+                         stdout=PIPE,stderr=STDOUT)
+        stdout = proc.stdout.read()
+        version_string = stdout.split('v')[1]
+        try:
+            version = tuple(map(int,version_string.split('.')))
+            pass_test = version in acceptable_version
+        except ValueError:
+            pass_test = False
+            version_string = stdout
+        self.assertTrue(pass_test,\
+         "Unsupported usearch version. %s is required, but running %s." \
+         % ('.'.join(map(str,acceptable_version)), version_string))
+         
+    def test_R_supported_version(self):
+        """R is in path and version is supported """
+        acceptable_version = [(2,12,0),(2,12,0)]
+        self.assertTrue(app_path('R'),
+         "usearch not found. This may or may not be a problem depending on "+\
+         "which components of QIIME you plan to use.")
+        command = "R --version | grep 'R version' | awk '{print $3}'"
+        proc = Popen(command,shell=True,universal_newlines=True,\
+                         stdout=PIPE,stderr=STDOUT)
+        stdout = proc.stdout.read()
+        version_string = stdout.strip()
+        try:
+            version = tuple(map(int,version_string.split('.')))
+            pass_test = version in acceptable_version
+        except ValueError:
+            pass_test = False
+            version_string = stdout
+        self.assertTrue(pass_test,\
+         "Unsupported R version. %s is required, but running %s." \
+         % ('.'.join(map(str,acceptable_version)), version_string))
+         
         
 def test_qiime_config_variable(variable, qiime_config, test,
                                access_var=R_OK, fail_on_missing=False):
@@ -616,6 +751,7 @@ def main():
      ("PyCogent version", pycogent_lib_version),
      ("NumPy version", numpy_lib_version),
      ("matplotlib version", matplotlib_lib_version),
+     ("biom-format version", biom_lib_version),
      ("QIIME library version", get_qiime_library_version()),
      ("QIIME script version", __version__),
      ("PyNAST version (if installed)", pynast_lib_version),
