@@ -6,7 +6,7 @@ __author__ = "Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME project"
 __credits__ = ["Greg Caporaso"]
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -19,13 +19,21 @@ from qiime.parse import parse_mapping_file
 from qiime.split_libraries_fastq import (process_fastq_single_end_read_file,
                                          BARCODE_DECODER_LOOKUP)
 from qiime.split_libraries import check_map
-from qiime.split_libraries_illumina import get_illumina_qual_chars
+from qiime.split_libraries_fastq import get_illumina_qual_chars
 from qiime.golay import get_invalid_golay_barcodes
 
 script_info = {}
 script_info['brief_description'] = "This script performs demultiplexing of Fastq sequence data where barcodes and sequences are contained in two separate fastq files (common on Illumina runs)."
 script_info['script_description'] = ""
-script_info['script_usage'] = [("Demultiplex and quality filter two lanes of Illumina run results and write results to ./sl_out/.","","%prog -i s_7_2_sequence.txt,s_8_2_sequence.txt -b s_7_1_sequence.txt,s_8_1_sequence.txt -o ./sl_out/ -m lane7_map.txt,lane8_map.txt")]
+
+script_info['script_usage'] = []
+
+script_info['script_usage'].append(("Demultiplex and quality filter (at Phred Q20) one lane of Illumina fastq data and write results to ./slout_q20.","","%prog -i lane1_read1.fastq.gz -b lane1_barcode.fastq.gz --rev_comp_mapping_barcodes -o slout_q20/ -m map.txt -q20"))
+
+script_info['script_usage'].append(("Demultiplex and quality filter (at Phred Q20) one lane of Illumina fastq data and write results to ./slout_q20. Store trimmed quality scores in addition to sequence data.","","%prog -i lane1_read1.fastq.gz -b lane1_barcode.fastq.gz --rev_comp_mapping_barcodes -o slout_q20/ -m map.txt --store_qual_scores -q20"))
+
+script_info['script_usage'].append(("Demultiplex and quality filter (at Phred Q20) two lanes of Illumina fastq data and write results to ./slout_q20.","","%prog -i lane1_read1.fastq.gz,lane2_read1.fastq.gz -b lane1_barcode.fastq.gz,lane2_barcode.fastq.gz --rev_comp_mapping_barcodes -o slout_q20/ -m map.txt,map.txt --store_qual_scores -q20"))
+
 script_info['output_description']= ""
 script_info['required_options'] = [\
  # Example required option
@@ -47,42 +55,50 @@ script_info['required_options'] = [\
              help='metadata mapping files (comma-separated if more than one)'),
 ]
 script_info['optional_options'] = [
+     make_option("--store_qual_scores",
+        default=False,
+        action='store_true',
+        help='store qual strings in .qual files [default: %default]'),
+     make_option("--store_demultiplexed_fastq",
+        default=False,
+        action='store_true',
+        help='write demultiplexed fastq files [default: %default]'),
      make_option("--retain_unassigned_reads",
         default=False,
         action='store_true',
         help='retain sequences which don\'t map to a barcode in the '+\
-        ' mapping file (sample ID will be "Unassigned") [default: %default]'),
+        'mapping file (sample ID will be "Unassigned") [default: %default]'),
      make_option('-r','--max_bad_run_length',type='int',\
         help='max number of consecutive low quality base calls allowed '+\
-        'before truncating a read [default: 1; the read is trucated at the'+\
-        'second low quality call]',default=1),\
-     make_option('-p','--min_per_read_length',type='int',\
-        help='min number of consecutive high quality base calls to include'+\
-        'a read (per single end read) [default: %default]',default=75),\
+        'before truncating a read [default: %default]',default=3),\
+     make_option('-p','--min_per_read_length_fraction',type='float',\
+        help='min number of consecutive high quality base calls to include '+\
+        'a read (per single end read) as a fraction of the input read length '+\
+        '[default: %default]',default=0.75),\
      make_option('-n','--sequence_max_n',type='int',\
         help='maximum number of N characters allowed in a sequence to retain it -- '
         'this is applied after quality trimming, and is total over combined paired '
         'end reads if applicable [default: %default]',default=0),
      make_option('-s','--start_seq_id',type='int',\
-        help='start seq_ids as ascending integers beginning with start_seq_id'+\
+        help='start seq_ids as ascending integers beginning with start_seq_id '+\
         '[default: %default]',default=0),\
      make_option('--rev_comp_barcode',action='store_true',\
-        help='reverse compliment barcode reads before lookup'+\
-        ' [default: %default]',default=False),\
+        help='reverse compliment barcode reads before lookup '+\
+        '[default: %default]',default=False),\
      make_option('--rev_comp_mapping_barcodes',action='store_true',\
-        help='reverse compliment barcode in mapping before lookup (useful if barcodes'+\
-        ' in mapping file are reverse compliments of golay codes)'+\
-        ' [default: %default]',default=False),\
+        help='reverse compliment barcode in mapping before lookup (useful if barcodes '+\
+        'in mapping file are reverse compliments of golay codes)'+\
+        '[default: %default]',default=False),\
      make_option('--rev_comp',action='store_true',\
-        help='reverse compliment sequence before writing to output file'+\
-        ' (useful for reverse-orientation reads) [default: %default]',
+        help='reverse compliment sequence before writing to output file '+\
+        '(useful for reverse-orientation reads) [default: %default]',
         default=False),\
-     make_option('--last_bad_quality_char',type='choice',
-        choices = list(get_illumina_qual_chars()),
-        help='the last character to be considered low quality '+\
-        '(i.e., these character and those before it will be considered '+\
-        'low quality base calls) [default: %default]',
-        default='B'),\
+     make_option('-q','--phred_quality_threshold',type='int',
+        help='the minimum acceptable Phred quality score (e.g., for Q20 and better, '+\
+        'specify -q 20) [default: %default]',default=3),
+     make_option('--last_bad_quality_char',
+      help='DEPRECATED: use -q instead. This method of setting is not robust to '+\
+      'different versions of CASAVA.'),\
      make_option('--barcode_type',type='string',
         help='The type of barcode used. This can be an integer, '+\
              'e.g. for length 6 barcodes, or golay_12 for golay '+\
@@ -110,11 +126,10 @@ def main():
     sequence_read_fps = opts.sequence_read_fps
     barcode_read_fps = opts.barcode_read_fps
     mapping_fps = opts.mapping_fps
-    
+    phred_quality_threshold = opts.phred_quality_threshold
     retain_unassigned_reads = opts.retain_unassigned_reads
+    min_per_read_length_fraction = opts.min_per_read_length_fraction
     max_bad_run_length = opts.max_bad_run_length
-    last_bad_quality_char = opts.last_bad_quality_char
-    min_per_read_length = opts.min_per_read_length
     rev_comp = opts.rev_comp
     rev_comp_barcode = opts.rev_comp_barcode
     rev_comp_mapping_barcodes = opts.rev_comp_mapping_barcodes
@@ -122,6 +137,16 @@ def main():
     start_seq_id = opts.start_seq_id
     # NEED TO FIX THIS FUNCTIONALITY - CURRENTLY READING THE WRONG FIELD
     filter_bad_illumina_qual_digit = False #opts.filter_bad_illumina_qual_digit
+    store_qual_scores = opts.store_qual_scores
+    store_demultiplexed_fastq = opts.store_demultiplexed_fastq
+    
+    if opts.last_bad_quality_char != None:
+        option_parser.error('--last_bad_quality_char is no longer supported. '
+         'Use -q instead (see option help text by passing -h)')
+    
+    if not (0 <= min_per_read_length_fraction <= 1):
+        option_parser.error('--min_per_read_length_fraction must be between '
+         '0 and 1 (inclusive). You passed %1.5f' % min_per_read_length_fraction)
     
     barcode_type = opts.barcode_type
     max_barcode_errors = opts.max_barcode_errors
@@ -131,8 +156,8 @@ def main():
     except KeyError:
         barcode_correction_fn = None
     
-    if len(sequence_read_fps) != len(barcode_read_fps):
-        parser.error("Same number of sequence and barcode files must be provided.")
+    if len(set([len(sequence_read_fps), len(barcode_read_fps), len(mapping_fps)])) > 1:
+        option_parser.error("Same number of sequence, barcode and mapping files must be provided.")
     
     output_dir = opts.output_dir
     create_dir(output_dir)
@@ -140,6 +165,33 @@ def main():
     output_fp_temp = '%s/seqs.fna.incomplete' % output_dir
     output_fp = '%s/seqs.fna' % output_dir
     output_f = open(output_fp_temp,'w')
+    qual_fp_temp = '%s/qual.fna.incomplete' % output_dir
+    qual_fp = '%s/seqs.qual' % output_dir
+    output_fastq_fp_temp = '%s/seqs.fastq.incomplete' % output_dir
+    output_fastq_fp = '%s/seqs.fastq' % output_dir
+    
+    if store_qual_scores:
+        qual_f = open(qual_fp_temp,'w')
+        # define a qual writer whether we're storing
+        # qual strings or not so we don't have to check
+        # every time through the for loop below
+        def qual_writer(h,q):
+            qual_f.write('>%s\n%s\n' % (h,q))
+    else:
+        def qual_writer(h,q):
+            pass
+    
+    if store_demultiplexed_fastq:
+        output_fastq_f = open(output_fastq_fp_temp,'w')
+        # define a fastq writer whether we're storing
+        # qual strings or not so we don't have to check
+        # every time through the for loop below
+        def fastq_writer(h,s,q):
+            output_fastq_f.write('@%s\n%s\n+\n%s\n' % (h,s,q))
+    else:
+        def fastq_writer(h,s,q):
+            pass
+    
     log_fp = '%s/split_library_log.txt' % output_dir
     log_f = open(log_fp,'w')
     histogram_fp = '%s/histograms.txt' % output_dir
@@ -179,8 +231,11 @@ def main():
             sequence_read_f = gzip_open(sequence_read_fp)
         else:
             sequence_read_f = open(sequence_read_fp,'U')
-
-        barcode_read_f = open(barcode_read_fp,'U')
+        
+        if barcode_read_fp.endswith('.gz'):
+            barcode_read_f = gzip_open(barcode_read_fp)
+        else:
+            barcode_read_f = open(barcode_read_fp,'U')
         seq_id = start_seq_id
         for fasta_header, sequence, quality, seq_id in \
             process_fastq_single_end_read_file(
@@ -189,8 +244,8 @@ def main():
                barcode_to_sample_id,
                store_unassigned=retain_unassigned_reads,
                max_bad_run_length=max_bad_run_length,
-               last_bad_quality_char=last_bad_quality_char,
-               min_per_read_length=min_per_read_length,
+               phred_quality_threshold=phred_quality_threshold,
+               min_per_read_length_fraction=min_per_read_length_fraction,
                rev_comp=rev_comp,
                rev_comp_barcode=rev_comp_barcode,
                seq_max_N=seq_max_N,
@@ -202,11 +257,23 @@ def main():
                barcode_correction_fn=barcode_correction_fn,
                max_barcode_errors=max_barcode_errors):
             output_f.write('>%s\n%s\n' % (fasta_header,sequence))
-        start_seq_id = seq_id + 1                                       
+            qual_writer(fasta_header,quality)
+            fastq_writer(fasta_header,sequence,quality)
+            
+        start_seq_id = seq_id + 1
         log_f.write('\n---\n\n')
         
     output_f.close()
     rename(output_fp_temp,output_fp)
+    
+    # process the optional output files, as necessary
+    if store_qual_scores:
+        qual_f.close()
+        rename(qual_fp_temp,qual_fp)
+    
+    if store_demultiplexed_fastq:
+        output_fastq_f.close()
+        rename(output_fastq_fp_temp,output_fastq_fp)
 
 if __name__ == "__main__":
     main()
