@@ -7,7 +7,7 @@ __author__ = "Jens Reeder"
 __copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Jens Reeder", "Rob Knight"]#remember to add yourself if you make changes
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Jens Reeder"
 __email__ = "jens.reeder@gmail.com"
 __status__ = "Release"
@@ -15,6 +15,7 @@ __status__ = "Release"
 from os import access, X_OK, R_OK
 from os.path import exists
 from subprocess import Popen, PIPE, STDOUT
+from cogent.util.misc import app_path
 from cogent.util.unit_test import TestCase, main
 from qiime.util import get_qiime_scripts_dir, load_qiime_config
 from qiime.denoiser.utils import get_flowgram_ali_exe
@@ -26,7 +27,11 @@ class DenoiserTests(TestCase):
 
         qiime_config = load_qiime_config()
         submit_script = qiime_config['cluster_jobs_fp']
+        
         if (submit_script):
+            full_path = app_path(submit_script)
+            if full_path:
+                submit_script = full_path
             self.assertTrue(exists(submit_script),
                             "cluster_jobs_fp is not set to a valid path in qiime config: %s" % submit_script)
             #check if executable
