@@ -6,11 +6,12 @@ __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Rob Knight", "Justin Kuczynski", "Greg Caporaso",\
                 "Cathy Lozupone", "Jens Reeder", "Daniel McDonald"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.4.0"
+__version__ = "1.5.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
 
+from biom.table import table_factory
 from numpy import array, nan
 from StringIO import StringIO
 from cogent.util.unit_test import TestCase,main
@@ -18,15 +19,15 @@ from cogent.util.misc import remove_files
 from qiime.util import get_tmp_filename
 from qiime.parse import (group_by_field, group_by_fields, 
     parse_distmat, parse_rarefaction_record, parse_rarefaction, parse_coords, 
-    parse_otu_table, make_envs_dict, fields_to_dict, parse_rarefaction_fname,
-    parse_qiime_parameters, parse_qiime_config_files,
+    parse_classic_otu_table, make_envs_dict, fields_to_dict, parse_rarefaction_fname,
+    parse_qiime_parameters, parse_qiime_config_files,sample_mapping_to_biom_table,
     parse_bootstrap_support, parse_sample_mapping, parse_distmat_to_dict,
     sample_mapping_to_otu_table, parse_taxonomy, parse_mapping_file, 
     parse_metadata_state_descriptions, parse_rarefaction_data,
     parse_illumina_line, parse_qual_score, parse_qual_scores, QiimeParseError,
     parse_newick,parse_trflp,parse_taxa_summary_table, parse_prefs_file,
     parse_mapping_file_to_dict, mapping_file_to_dict, MinimalQualParser,
-    parse_denoiser_mapping)
+    parse_denoiser_mapping,parse_otu_map,parse_taxonomy_to_otu_metadata)
 
 class TopLevelTests(TestCase):
     """Tests of top-level functions"""
@@ -445,12 +446,12 @@ eigvals\t4.94\t1.79\t1.50
         self.assertEqual(obs, exp)
 
     
-    def test_parse_otu_table_legacy(self):
-        """parse_otu_table functions as expected with legacy OTU table
+    def test_parse_classic_otu_table_legacy(self):
+        """parse_classic_otu_table functions as expected with legacy OTU table
         """
         data = self.legacy_otu_table1
         data_f = (data.split('\n'))
-        obs = parse_otu_table(data_f)
+        obs = parse_classic_otu_table(data_f)
         exp = (['Fing','Key','NA'],
                ['0','1','2','3','4'],
                array([[19111,44536,42],[1216,3500,6],[1803,1184,2],
@@ -458,12 +459,12 @@ eigvals\t4.94\t1.79\t1.50
                self.expected_lineages1)
         self.assertEqual(obs, exp)
         
-    def test_parse_otu_table(self):
-        """parse_otu_table functions as expected with new-style OTU table
+    def test_parse_classic_otu_table(self):
+        """parse_classic_otu_table functions as expected with new-style OTU table
         """
         data = self.otu_table1
         data_f = (data.split('\n'))
-        obs = parse_otu_table(data_f)
+        obs = parse_classic_otu_table(data_f)
         exp = (['Fing','Key','NA'],
                ['0','1','2','3','4'],
                array([[19111,44536,42],[1216,3500,6],[1803,1184,2],
@@ -471,13 +472,13 @@ eigvals\t4.94\t1.79\t1.50
                self.expected_lineages1)
         self.assertEqual(obs, exp)
     
-    def test_parse_otu_table_floats_in_table(self):
-        """parse_otu_table functions using an OTU table containing floats
+    def test_parse_classic_otu_table_floats_in_table(self):
+        """parse_classic_otu_table functions using an OTU table containing floats
            but cast as int....this will automatically cast into floats"""
            
         data = self.otu_table1_floats
         data_f = (data.split('\n'))
-        obs = parse_otu_table(data_f)
+        obs = parse_classic_otu_table(data_f)
         exp = (['Fing','Key','NA'],
                ['0','1','2','3','4'],
                array([[19111.0,44536.0,42.0],[1216.0,3500.0,6.0],
@@ -486,8 +487,8 @@ eigvals\t4.94\t1.79\t1.50
                self.expected_lineages1)
         self.assertEqual(obs, exp)  
         
-    def test_parse_otu_table_float_counts(self):
-        """parse_otu_table should return correct result from small table"""
+    def test_parse_classic_otu_table_float_counts(self):
+        """parse_classic_otu_table should return correct result from small table"""
         data = """#Full OTU Counts
 #OTU ID	Fing	Key	NA	Consensus Lineage
 0	19111	44536	42	Bacteria; Actinobacteria; Actinobacteridae; Propionibacterineae; Propionibacterium
@@ -496,7 +497,7 @@ eigvals\t4.94\t1.79\t1.50
 3	1722	4903	17	Bacteria; Firmicutes; Alicyclobacillaceae; Bacilli; Staphylococcaceae
 4	589	2074	34	Bacteria; Cyanobacteria; Chloroplasts; vectors"""
         data_f = (data.split('\n'))
-        obs = parse_otu_table(data_f,count_map_f=float)
+        obs = parse_classic_otu_table(data_f,count_map_f=float)
         exp = (['Fing','Key','NA'],
                ['0','1','2','3','4'],
                array([[19111.,44536.,42.],[1216.,3500.,6.],[1803.,1184.,2.],\
@@ -508,8 +509,8 @@ eigvals\t4.94\t1.79\t1.50
                 ['Bacteria','Cyanobacteria','Chloroplasts','vectors']])
         self.assertEqual(obs, exp)
         
-    def test_parse_otu_table_file(self):
-        """parse_otu_table should return correct result on fileio format object"""
+    def test_parse_classic_otu_table_file(self):
+        """parse_classic_otu_table should return correct result on fileio format object"""
         data = """#Full OTU Counts
 #OTU ID	Fing	Key	NA	Consensus Lineage
 0	19111	44536	42	Bacteria; Actinobacteria; Actinobacteridae; Propionibacterineae; Propionibacterium
@@ -518,7 +519,7 @@ eigvals\t4.94\t1.79\t1.50
 3	1722	4903	17	Bacteria; Firmicutes; Alicyclobacillaceae; Bacilli; Staphylococcaceae
 4	589	2074	34	Bacteria; Cyanobacteria; Chloroplasts; vectors"""
         data_f = StringIO(data)
-        obs = parse_otu_table(data_f)
+        obs = parse_classic_otu_table(data_f)
         exp = (['Fing','Key','NA'],
                ['0','1','2','3','4'],
                array([[19111,44536,42],[1216,3500,6],[1803,1184,2],\
@@ -530,8 +531,8 @@ eigvals\t4.94\t1.79\t1.50
                 ['Bacteria','Cyanobacteria','Chloroplasts','vectors']])
         self.assertEqual(obs, exp)
         
-    def test_parse_otu_table_consensus_lineage(self):
-        """parse_otu_table should accept 'consensusLineage'"""
+    def test_parse_classic_otu_table_consensus_lineage(self):
+        """parse_classic_otu_table should accept 'consensusLineage'"""
         data = """#Full OTU Counts
 #OTU ID	Fing	Key	NA	consensusLineage
 0	19111	44536	42	Bacteria; Actinobacteria; Actinobacteridae; Propionibacterineae; Propionibacterium
@@ -540,7 +541,7 @@ eigvals\t4.94\t1.79\t1.50
 3	1722	4903	17	Bacteria; Firmicutes; Alicyclobacillaceae; Bacilli; Staphylococcaceae
 4	589	2074	34	Bacteria; Cyanobacteria; Chloroplasts; vectors"""
         data_f = StringIO(data)
-        obs = parse_otu_table(data_f)
+        obs = parse_classic_otu_table(data_f)
         exp = (['Fing','Key','NA'],
                ['0','1','2','3','4'],
                array([[19111,44536,42],[1216,3500,6],[1803,1184,2],\
@@ -616,6 +617,15 @@ eigvals\t4.94\t1.79\t1.50
         self.assertEqual(result, ['#Full OTU Counts',\
          '#OTU ID\tsample1\tsample2\tsample3', 'OTU2\t1\t2\t0', \
         'OTU1\t3\t0\t2'])
+
+    def test_sample_mapping_to_biom_table(self):
+        """sample_mapping_to_biom_table works"""
+        lines = self.SampleMapping
+        actual = sample_mapping_to_biom_table(lines)
+        exp = table_factory(array([[3.,0.,2.],[1.,2.,0.]]),
+                            ['sample1','sample2','sample3'],
+                            ['OTU1','OTU2'])
+        self.assertEqual(actual.sortBySampleId(), exp.sortBySampleId())
     
     def test_parse_taxonomy(self):
         """ should parse taxonomy example, keeping otu id only"""
@@ -631,6 +641,60 @@ eigvals\t4.94\t1.79\t1.50
          "Root;Bacteria;Firmicutes;\"Clostridia\";Clostridiales")
         self.assertEqual(res['338'],
          "Root;Bacteria")
+
+    def test_parse_taxonomy_to_otu_metadata(self):
+        """parsing of taxonomy file to otu metadata format functions as expected
+        """
+        example_tax = \
+"""412 PC.635_647	Root;Bacteria;Firmicutes;"Clostridia";Clostridiales	0.930
+319 PC.355_281	Root;Bacteria;Bacteroidetes	0.970
+353 PC.634_154	Root;Bacteria;Bacteroidetes	0.830
+17 PC.607_302	Root;Bacteria;Bacteroidetes	0.960
+13	Root;Bacteria;Firmicutes;"Clostridia";Clostridiales	0.870
+338 PC.593_1314	Root;Bacteria	0.990"""
+        actual = parse_taxonomy_to_otu_metadata(example_tax.split('\n'))
+        expected = {'412':{'taxonomy':['Root','Bacteria','Firmicutes','"Clostridia"','Clostridiales'],'score':0.930},
+ '319':{'taxonomy':['Root','Bacteria','Bacteroidetes'],'score':0.970},
+ '353':{'taxonomy':['Root','Bacteria','Bacteroidetes'],'score':0.830},
+ '17':{'taxonomy':['Root','Bacteria','Bacteroidetes'],'score':0.960},
+ '13':{'taxonomy':['Root','Bacteria','Firmicutes','"Clostridia"','Clostridiales'],'score':0.870},
+ '338':{'taxonomy':['Root','Bacteria'],'score':0.990}}
+        self.assertEqual(actual,expected)
+
+    def test_parse_taxonomy_to_otu_metadata_alt_labels(self):
+        """parsing of taxonomy file to otu metadata format functions as expected
+        """
+        def f(v):
+            return 1. + float(v)
+        example_tax = \
+"""412 PC.635_647	0.0
+319 PC.355_281	0.970
+353 PC.634_154	0.830
+17 PC.607_302	0.960
+13	0.870
+338 PC.593_1314	0.990"""
+        actual = parse_taxonomy_to_otu_metadata(example_tax.split('\n'),labels=['something'],process_fs=[f])
+        expected = {'412':{'something':1.0},
+ '319':{'something':1.970},
+ '353':{'something':1.830},
+ '17':{'something':1.960},
+ '13':{'something':1.870},
+ '338':{'something':1.990}}
+        self.assertEqual(actual,expected)
+
+
+
+    def test_parse_taxonomy_to_otu_metadata_invalid_input(self):
+        """parsing of taxonomy file to otu metadata format functions as expected
+        """
+        example_tax = \
+"""412 PC.635_647	Root;Bacteria;Firmicutes;"Clostridia";Clostridiales	0.930
+319 PC.355_281	Root;Bacteria;Bacteroidetes	0.970
+353 PC.634_154	Root;Bacteria;Bacteroidetes	0.830
+17 PC.607_302	Root;Bacteria;Bacteroidetes	0.960
+13	Root;Bacteria;Firmicutes;"Clostridia";Clostridiales	0.870
+338 PC.593_1314	Root;Bacteria	0.990	42556	Additional fields raise error"""
+        self.assertRaises(ValueError,parse_taxonomy_to_otu_metadata,example_tax.split('\n'))
  
     def test_parse_qiime_config_files(self):
         """ parse_qiime_config_files functions as expected """
@@ -839,7 +903,37 @@ Sample 5	25
                     'Read2':['Read2'],
                     'Read3':['Read3','Read6']}
         self.assertEqual(actual,expected)
-        
+    
+    def test_parse_otu_map(self):
+        """ parse_otu_map functions as expected 
+        """
+        otu_map_f = """otu1	s1_0	s2_1	s1_99
+2	s1_9	s5_2 comment	s3_99	1_3	s1_75
+otu3	s8_7	s2_5""".split('\n')
+        expected_map = {(0,0):2,(0,1):1,
+                        (1,0):2,(1,2):1,(1,3):1,(1,4):1,
+                        (2,5):1,(2,1):1}
+        expected_sids = ['s1' , 's2', 's5','s3','1','s8']
+        expected_oids = ['otu1','2','otu3']
+        actual = parse_otu_map(otu_map_f)
+        self.assertEqual(actual[0],expected_map)
+        self.assertEqual(actual[1],expected_sids)
+        self.assertEqual(actual[2],expected_oids)
+
+    def test_parse_otu_map_w_excludes(self):
+        """ parse_otu_map functions as expected when excluding otu ids
+        """
+        otu_map_f = """otu1	s1_0	s2_1	s1_99
+2	s1_9	s5_2 comment	s3_99	1_3	s1_75
+otu3	s8_7	s2_5""".split('\n')
+        excludes = ['otu1','2']
+        expected_map = {(0,0):1,(0,1):1}
+        expected_sids = ['s8','s2']
+        expected_oids = ['otu3']
+        actual = parse_otu_map(otu_map_f,excludes)
+        self.assertEqual(actual[0],expected_map)
+        self.assertEqual(actual[1],expected_sids)
+        self.assertEqual(actual[2],expected_oids)
 
 illumina_read1 = """HWI-6X_9267:1:1:4:1699#ACCACCC/1:TACGGAGGGTGCGAGCGTTAATCGCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCGAAAAAAAAAAAAAAAAAAAAAAA:abbbbbbbbbb`_`bbbbbb`bb^aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaDaabbBBBBBBBBBBBBBBBBBBB
 HWI-6X_9267:1:1:4:390#ACCTCCC/1:GACAGGAGGAGCAAGTGTTATTCAAATTATGCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCGGGGGGGGGGGGGGGAAAAAAAAAAAAAAAAAAAAAAA:aaaaaaaaaa```aa\^_aa``aVaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaBaaaaa""".split('\n')
