@@ -4,6 +4,14 @@
 provides unit tests for the usearch.py module
 """
 
+__author__ = "William Walters"
+__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__credits__ = ["William Walters", "Jose Carlos Clemente Litran"]
+__license__ = "GPL"
+__version__ = "1.5.0"
+__maintainer__ = "William Walters"
+__email__ = "william.a.walters@colorado.edu"
+__status__ = "Release"
 
 from os.path import isfile, basename
 from shutil import rmtree
@@ -17,10 +25,11 @@ from qiime.util import create_dir
 
 from qiime.pycogent_backports.usearch import (Usearch,
  clusters_from_blast_uc_file, usearch_fasta_sort_from_filepath,
- usearch_dereplicate_exact_subseqs, usearch_sort_by_abundance,
+ usearch_dereplicate_exact_subseqs, usearch_dereplicate_exact_seqs,
+ usearch_sort_by_abundance,
  usearch_cluster_error_correction, usearch_chimera_filter_de_novo,
  usearch_chimera_filter_ref_based, usearch_cluster_seqs,
- enumerate_otus, assign_reads_to_otus, otu_pipe, concatenate_fastas,
+ enumerate_otus, assign_reads_to_otus, usearch_qf, concatenate_fastas,
  get_retained_chimeras)
 
 __author__ = "William Walters"
@@ -76,6 +85,8 @@ class UsearchTests(TestCase):
          expected_retained_chimeras_union
         self.expected_retained_chimeras_intersection =\
          expected_retained_chimeras_intersection
+        self.expected_derep_seqs_full_len =\
+         expected_derep_seqs_full_len
         
         # Create temporary files for use with unit tests
         
@@ -143,6 +154,7 @@ class UsearchTests(TestCase):
         seq_file = open(self.tmp_dna_seqs_ref_otu_picking, "w")
         seq_file.write(self.dna_seqs_reference_otu_picking)
         seq_file.close()
+        
        
         self._files_to_remove =\
          [self.tmp_seq_filepath1, self.tmp_seq_filepath2,
@@ -159,11 +171,11 @@ class UsearchTests(TestCase):
             for curr_dir in self._dirs_to_remove:
                 rmtree(curr_dir)
             
-    def test_otu_pipe(self):
+    def test_usearch_qf(self):
         """ Main program loop test, with default parameters """
         
         # cluster size filtering set to 1 instead of default 4
-        clusters, failures = otu_pipe(self.tmp_seq_filepath2,
+        clusters, failures = usearch_qf(self.tmp_seq_filepath2,
                                       output_dir = self.tmp_dir,
                                       db_filepath = self.tmp_ref_database,
                                       minsize = 1,
@@ -177,11 +189,11 @@ class UsearchTests(TestCase):
         self.assertEqual(clusters, expected_clusters)
         self.assertEqual(failures, expected_failures)
         
-    def test_otu_pipe_reference_otu_picking(self):
+    def test_usearch_qf_reference_otu_picking(self):
         """ Main program loop test, with reference + new clusters """
         
         # cluster size filtering set to 1 instead of default 4
-        clusters, failures = otu_pipe(self.tmp_dna_seqs_ref_otu_picking,
+        clusters, failures = usearch_qf(self.tmp_dna_seqs_ref_otu_picking,
                                       output_dir = self.tmp_dir,
                                       refseqs_fp = self.tmp_ref_database,
                                       reference_chimera_detection=False,
@@ -198,11 +210,11 @@ class UsearchTests(TestCase):
         self.assertEqual(clusters, expected_clusters)
         self.assertEqual(failures, expected_failures)
     
-    def test_otu_pipe_reference_otu_picking_no_new_clusters(self):
+    def test_usearch_qf_reference_otu_picking_no_new_clusters(self):
         """ Main program loop test, with reference and no new clusters """
         
         # cluster size filtering set to 1 instead of default 4
-        clusters, failures = otu_pipe(self.tmp_dna_seqs_ref_otu_picking,
+        clusters, failures = usearch_qf(self.tmp_dna_seqs_ref_otu_picking,
                                       output_dir = self.tmp_dir,
                                       refseqs_fp = self.tmp_ref_database,
                                       reference_chimera_detection=False,
@@ -219,11 +231,11 @@ class UsearchTests(TestCase):
         self.assertEqual(failures, expected_failures)
         
         
-    def test_otu_pipe_no_ref_database(self):
+    def test_usearch_qf_no_ref_database(self):
         """ Main program loop with no reference chimera testing """
         
         # cluster size filtering set to 1 instead of default 4
-        clusters, failures = otu_pipe(self.tmp_seq_filepath2,
+        clusters, failures = usearch_qf(self.tmp_seq_filepath2,
                                       output_dir = self.tmp_dir,
                                       reference_chimera_detection=False,
                                       minsize = 1,
@@ -239,11 +251,11 @@ class UsearchTests(TestCase):
         self.assertEqual(clusters, expected_clusters)
         self.assertEqual(failures, expected_failures)
         
-    def test_otu_pipe_union(self):
+    def test_usearch_qf_union(self):
         """ Main program loop with union nonchimera retention """
         
         # cluster size filtering set to 1 instead of default 4
-        clusters, failures = otu_pipe(self.tmp_seq_filepath2,
+        clusters, failures = usearch_qf(self.tmp_seq_filepath2,
                                       output_dir = self.tmp_dir,
                                       reference_chimera_detection=False,
                                       minsize = 1,
@@ -261,11 +273,11 @@ class UsearchTests(TestCase):
         self.assertEqual(failures, expected_failures)
 
         
-    def test_otu_pipe_disabled_filters(self):
+    def test_usearch_qf_disabled_filters(self):
         """ Returns expected clustering with no filtering """
         
         # cluster size filtering set to 1 instead of default 4
-        clusters, failures = otu_pipe(self.tmp_seq_filepath2,
+        clusters, failures = usearch_qf(self.tmp_seq_filepath2,
                                       output_dir = self.tmp_dir,
                                       de_novo_chimera_detection=False,
                                       reference_chimera_detection=False,
@@ -282,7 +294,7 @@ class UsearchTests(TestCase):
         self.assertEqual(clusters, expected_clusters)
         self.assertEqual(failures, expected_failures)
         
-    def test_otu_pipe_generates_logs(self):
+    def test_usearch_qf_generates_logs(self):
         """ Generates expected log files """
         
         curr_output_dir = get_tmp_filename(prefix='/UsearchLogTest_',suffix='/')
@@ -293,7 +305,7 @@ class UsearchTests(TestCase):
         self._dirs_to_remove.append(curr_output_dir)
         
         # cluster size filtering set to 1 instead of default 4
-        clusters, failures = otu_pipe(self.tmp_seq_filepath2,
+        clusters, failures = usearch_qf(self.tmp_seq_filepath2,
                                       output_dir = curr_output_dir,
                                       db_filepath = self.tmp_ref_database,
                                       minsize = 1,
@@ -539,8 +551,23 @@ class UsearchTests(TestCase):
         self._files_to_remove.append(output_filepath)
         
         actual_seqs = [line.strip() for line in open(output_filepath, "U")]
+
         
         self.assertEqual(actual_seqs, self.expected_derep_seqs)
+
+    def test_usearch_dereplicate_exact_seqs(self):
+        """ Properly dereplicates fasta file """
+
+        app_result, output_filepath =\
+                    usearch_dereplicate_exact_seqs(self.tmp_dna_seqs_with_dups,
+                                                      remove_usearch_logs=True,
+                                                      working_dir=self.tmp_dir)
+
+        self._files_to_remove.append(output_filepath)
+
+        actual_seqs = [line.strip() for line in open(output_filepath, "U")]
+
+        self.assertEqual(actual_seqs, self.expected_derep_seqs_full_len)
         
     def test_usearch_fasta_sort_from_filepath(self):
         """ Properly sorts fasta according to seq length """
@@ -764,11 +791,13 @@ GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTTAT
 GCGGTATTAGCACCTATTTCTAAGTGTTATCCCCCAGTATACGGCAGGTTCTCCACGCGTTACT"""
 
 dna_seqs_with_dups=""">seq1
-GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTTAT
+GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTTAA
 >seq2
 TTGGGCCGTGTCTCAGTCCCAATGTGGCCGTCACCCTCTCAGGCCGGCTACTGATCGTCGCCTTGGTGGGCCTTTACCCC
 >seq3
-GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTTAT"""
+TTGGGCCGTGTCTCAGTCCCAATGTGGCCGTCACCCTCTCAGGCCGGCTACTGATCGTCGCCTTGGTGGGCCTTTACCCC
+>seq4
+GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTT"""
 
 # Expected output file data
 uc_lines1 = """# usearch --id 0.97 --uc usearch_picked_otus/assign_reads_to_otus.uc --query seqs.fna --global --db usearch_picked_otus/enumerated_otus.fasta
@@ -793,9 +822,9 @@ expected_otu_assignments = """# Tab-separated fields:
 # For C and D types, PctId is average id with seed.
 # QueryStart and SeedStart are zero-based relative to start of sequence.
 # If minus strand, SeedStart is relative to reverse-complemented seed.
-H\t2\t199\t97.5\t+\t0\t0\t119M80D\tref1 ecoli sequence\tusearch_ecoli_seq2
+H\t2\t199\t97.5\t.\t0\t0\t119M80D\tref1 ecoli sequence\tusearch_ecoli_seq2
 N\t*\t178\t*\t*\t*\t*\t*\tEU199232 1 1236 Bacteria/Deltaproteobacteria/Desulfurella - Hippea/uncultured\t*
-H\t1\t180\t100.0\t+\t0\t0\t97M83D\tL07864 1 1200 Bacteria/Beta Gammaproteobacteria/Solemya symbiont\tSolemya seq""".split('\n')
+H\t1\t180\t100.0\t.\t0\t0\t97M83D\tL07864 1 1200 Bacteria/Beta Gammaproteobacteria/Solemya symbiont\tSolemya seq""".split('\n')
 
 expected_enumerated_fasta = """>0
 AACCCCCACGGTGGATGCCACACGCCCCATACAAAGGGTAGGATGCTTAAGACACATCGCGTCAGGTTTGTGTCAGGCCT
@@ -997,7 +1026,7 @@ CTGGGCCGTGTCTCAGTCCCAGTGTGGCCGTCCGCCCTCTCAGGTCAGCTACTGATCGTCGCCTTGGTAGGCCATTACCC
 TACCAACTAGCTAATCAGACGCGAGGCCATCTCTCAGCGATAAATCTTTGATATATCTGCCATGCGACAAACATATATTA
 TGCGGTATTAGCAGTCGTTTCCAACTGTTGTCCCCCTCTGAAAGGCAGGTT""".split('\n')
 
-expected_cluster_err_seqs = """>Cluster0;size=326
+expected_cluster_err_seqs = """>Cluster0;size=2
 AACCCCCACGGTGGATGCCACACGCCCCATACAAAGGGTAGGATGCTTAAGACACATCGCGTCAGGTTTGTGTCAGGCCT
 AACCCCCACGGTGGATGCCACACGCCCCATACAAAGGGTAGGATGCTTAAGACACATCGCGTCAGGTTTGTGTCAGGCCT""".split('\n')
 
@@ -1101,10 +1130,17 @@ CGCCGACTAGCTAATGCGCCGCATGGCCATCCGCAGCCGATAAATCTTTAAACATCGGGAGATGCCTCCCAACGTTGTTA
 CGCGGTATTAGACGGAATTTCTTCCGCTTATCCCCCTGCTGCGGGCAGGTTCCATACGTGTTACTCACCCGTGCGCCGG""".split('\n')
 
 expected_derep_seqs = """>seq1;size=2
-GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTTAT
->seq2;size=1
+GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTTAA
+>seq2;size=2
 TTGGGCCGTGTCTCAGTCCCAATGTGGCCGTCACCCTCTCAGGCCGGCTACTGATCGTCGCCTTGGTGGGCCTTTACCCC""".split('\n')
 
+expected_derep_seqs_full_len = """>Cluster0;size=1
+GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTTAA
+>Cluster1;size=2
+TTGGGCCGTGTCTCAGTCCCAATGTGGCCGTCACCCTCTCAGGCCGGCTACTGATCGTCGCCTTGGTGGGCCTTTACCCC
+>Cluster2;size=1
+GCCAACCAGCTAATCAGACGCGGGTCCATCTTGCACCACCGGAGTTTTTCACACTGCTTCATGCGAAGCTGTGCGCTT""".split('\n')
+
 expected_len_sorted_seqs = """>chimera
 CGCGTGTATGAAGAAGGCCTTCGGGTTGTAAAGTACTTTCAGCGGGGAGGAGGGAGTAAAGTTAATACCTTTGCTCATTGACCCCTAGGGTGGGAATAACCCGGGGAAACCCGGGCTAATACCGAATAAGACCACAGGAGGCGACTCCAGAGGGTCAAAGGGAGCCTTGGCCTCCCCC
 >usearch_ecoli_seq2
